@@ -10,8 +10,8 @@ from __future__ import with_statement
 import math, os, re, sys
 
 import colormath
-
-from safe_print import safe_print
+import x3dom
+from log import safe_print
 from util_io import GzipFileProper, StringIOu as StringIO
 
 
@@ -28,8 +28,12 @@ def rpad(value, width):
 	
 	value is converted to a string first.
 	
+	If value wasn't a number originally, return the quoted string.
+	
 	"""
 	strval = str(value)
+	if not isinstance(value, (int, float, long, complex)):
+		return '"%s"' % strval
 	i = strval.find(".")
 	if i > -1:
 		if i < width - 1:
@@ -45,8 +49,12 @@ def rcut(value, width):
 	
 	value is converted to a string first.
 	
+	If value wasn't a number originally, return the quoted string.
+	
 	"""
 	strval = str(value)
+	if not isinstance(value, (int, float, long, complex)):
+		return '"%s"' % strval
 	i = strval.find(".")
 	if i > -1:
 		if i < width - 1:
@@ -73,6 +81,32 @@ def sort_RGB_white_to_top(a, b):
 		return 0
 
 
+def sort_by_HSI(a, b):
+	a = list(colormath.RGB2HSI(*a[:3]))
+	b = list(colormath.RGB2HSI(*b[:3]))
+	a[0] = round(a[0], 12)
+	b[0] = round(b[0], 12)
+	if a > b:
+		return 1
+	elif a < b:
+		return -1
+	else:
+		return 0
+
+
+def sort_by_HSL(a, b):
+	a = list(colormath.RGB2HSL(*a[:3]))
+	b = list(colormath.RGB2HSL(*b[:3]))
+	a[0] = round(a[0], 12)
+	b[0] = round(b[0], 12)
+	if a > b:
+		return 1
+	elif a < b:
+		return -1
+	else:
+		return 0
+
+
 def sort_by_HSV(a, b):
 	a = list(colormath.RGB2HSV(*a[:3]))
 	b = list(colormath.RGB2HSV(*b[:3]))
@@ -537,6 +571,20 @@ class CGATS(dict):
 		valueslist.sort(sort_RGB_white_to_top)
 		return data.set_RGB_XYZ_values(valueslist)
 	
+	def sort_by_HSI(self):
+		data, valueslist = self.get_RGB_XYZ_values()
+		if not valueslist:
+			return False
+		valueslist.sort(sort_by_HSI)
+		return data.set_RGB_XYZ_values(valueslist)
+	
+	def sort_by_HSL(self):
+		data, valueslist = self.get_RGB_XYZ_values()
+		if not valueslist:
+			return False
+		valueslist.sort(sort_by_HSL)
+		return data.set_RGB_XYZ_values(valueslist)
+	
 	def sort_by_HSV(self):
 		data, valueslist = self.get_RGB_XYZ_values()
 		if not valueslist:
@@ -752,19 +800,36 @@ class CGATS(dict):
 											 self.type)
 		return context
 	
-	def export_vrml(self, filename, devicelocations=True, RGB_black_offset=40,
-					normalize_RGB_white=False, compress=True):
+	def export_3d(self, filename, colorspace="RGB", RGB_black_offset=40,
+				  normalize_RGB_white=False, compress=True, format="VRML"):
+		if colorspace not in ("DIN99", "DIN99b", "DIN99c", "DIN99d", "LCH(ab)",
+							  "LCH(uv)", "Lab", "Luv", "Lu'v'", "RGB", "xyY",
+							  "HSI", "HSL", "HSV"):
+			raise ValueError("export_3d: Unknown colorspace %r" % colorspace)
 		data = self.queryv1("DATA")
 		if self.queryv1("ACCURATE_EXPECTED_VALUES") == "true":
 			cat = "Bradford"
 		else:
 			cat = "XYZ scaling"
 		radius = 15.0 / (len(data) ** (1.0 / 3.0))
+		scale = 1.0
+		if colorspace.startswith("DIN99"):
+			if colorspace == "DIN99":
+				scale = 100.0 / 40
+			else:
+				scale = 100.0 / 50
+			radius /= scale
 		white = data.queryi1({"RGB_R": 100, "RGB_G": 100, "RGB_B": 100})
 		if white:
-			white = colormath.get_whitepoint((white["XYZ_X"],
-											  white["XYZ_Y"],
-											  white["XYZ_Z"]))
+			white = white["XYZ_X"], white["XYZ_Y"], white["XYZ_Z"]
+		else:
+			white = "D50"
+		white = colormath.get_whitepoint(white)
+		d50 = colormath.get_whitepoint("D50")
+		if colorspace == "Lu'v'":
+			white_u_, white_v_ = colormath.XYZ2Lu_v_(*d50)[1:]
+		elif colorspace == "xyY":
+			white_x, white_y = colormath.XYZ2xyY(*d50)[:2]
 		vrml = """#VRML V2.0 utf8
 
 Transform {
@@ -780,10 +845,11 @@ Transform {
 		}
 
 		Viewpoint {
-			position 0 0 340
+			fieldOfView %(fov)s
+			position 0 0 %(z)s
 		}
 
-		#%(axes)s
+		%(axes)s
 %(children)s
 	]
 }
@@ -799,15 +865,95 @@ Transform {
 			]
 		}
 """
-		if devicelocations:
-			axes = ""
+		axes = ""
+		if (colorspace not in ("Lab", "Luv") and
+			not colorspace.startswith("DIN99")):
+			if colorspace in ("Lu'v'", "xyY"):
+				maxz = scale = 100
+				maxxy = 200
+				radius /= 2.0
+				if colorspace == "Lu'v'":
+					xlabel, ylabel, zlabel = "u' 0.6", "v' 0.6", "L* 100"
+					offsetx, offsety = -.3, -.3
+					scale = maxxy / .6
+				else:
+					xlabel, ylabel, zlabel = "x 0.8", "y 0.8", "Y 100"
+					offsetx, offsety = -.4, -.4
+					scale = maxxy / .8
+				axes = x3dom.get_vrml_axes(xlabel, ylabel, zlabel,
+										   offsetx * scale, offsety * scale,
+										   0, maxxy, maxxy, maxz)
+			elif colorspace in ("LCH(ab)", "LCH(uv)"):
+				if colorspace == "LCH(ab)":
+					xlabel, ylabel, zlabel = "H(ab)", "C(ab)", "L*"
+				else:
+					xlabel, ylabel, zlabel = "H(uv)", "C(uv)", "L*"
+				axes = x3dom.get_vrml_axes(xlabel, ylabel, zlabel,
+										   -180, -100, 0, 360, 200, 100, False)
 		else:
-			axes = """# L* axis
+			if colorspace.startswith("DIN99"):
+				axes += """Transform {
+			translation %.1f %.1f -50.0
+			children [
+				Shape {
+					geometry Text {
+						string ["%s"]
+						fontStyle FontStyle { family "SANS" style "BOLD" size %.1f }
+					}
+					appearance Appearance {
+						material Material { diffuseColor 0.7 0.7 0.7 }
+					}
+				}
+			]
+		}
+""" % (100 / scale, 100 / scale, colorspace, 10.0 / scale)
+				(pxlabel,
+				 nxlabel,
+				 pylabel,
+				 nylabel,
+				 pllabel) = ('"a", "+%i"' % (100 / scale),
+							 '"a", "-%i"' % (100 / scale),
+							 '"b +%i"' % (100 / scale),
+							 '"b -%i"' % (100 / scale),
+							 '"L", "+100"')
+			else:
+				if colorspace == "Luv":
+					x = "u"
+					y = "v"
+				else:
+					x = "a"
+					y = "b"
+				(pxlabel,
+				 nxlabel,
+				 pylabel,
+				 nylabel,
+				 pllabel) = ('"%s*", "+100"' % x,
+							 '"%s*", "-100"' % x,
+							 '"%s* +100"' % y,
+							 '"%s* -100"' % y,
+							 '"L*", "+100"')
+			values = {"wh": 2.0 / scale,
+					  "ab": 100.0 / scale,
+					  "aboffset": 50.0 / scale,
+					  "fontsize": 10.0 / scale,
+					  "ap": 102.0 / scale,
+					  "an": 108.0 / scale,
+					  "Ln": 3.0,
+					  "bp0": 3.0,
+					  "bp1": 103.0 / scale,
+					  "bn0": 3.0,
+					  "bn1": 107.0 / scale,
+					  "pxlabel": pxlabel,
+					  "nxlabel": nxlabel,
+					  "pylabel": pylabel,
+					  "nylabel": nylabel,
+					  "pllabel": pllabel}
+			axes += """# L* axis
 		Transform {
 			translation 0.0 0.0 0.0
 			children [
 				Shape {
-					geometry Box { size 2.0 2.0 100.0 }
+					geometry Box { size %(wh).1f %(wh).1f 100.0 }
 					appearance Appearance {
 						material Material { diffuseColor 0.7 0.7 0.7 }
 					}
@@ -816,12 +962,12 @@ Transform {
 		}
 		# L* axis label
 		Transform {
-			translation -3.0 -2.0 55.0
+			translation -%(Ln).1f -%(wh).1f 55.0
 			children [
 				Shape {
 					geometry Text {
-						string ["L"]
-						fontStyle FontStyle { family "SANS" style "BOLD" size 10.0 }
+						string [%(pllabel)s]
+						fontStyle FontStyle { family "SANS" style "BOLD" size %(fontsize).1f }
 					}
 					appearance Appearance {
 						material Material { diffuseColor 0.7 0.7 0.7}
@@ -829,26 +975,26 @@ Transform {
 				}
 			]
 		}
-		# +a axis
+		# +x axis
 		Transform {
-			translation 50.0 0.0 -50.0
+			translation %(aboffset).1f 0.0 -50.0
 			children [
 				Shape {
-					geometry Box { size 100.0 2.0 2.0 }
+					geometry Box { size %(ab).1f %(wh).1f %(wh).1f }
 					appearance Appearance {
 						material Material { diffuseColor 1.0 0.0 0.0 }
 					}
 				}
 			]
 		}
-		# +a axis label
+		# +x axis label
 		Transform {
-			translation 102.0 -2.0 -50.0
+			translation %(ap).1f -%(wh).1f -50.0
 			children [
 				Shape {
 					geometry Text {
-						string ["+a"]
-						fontStyle FontStyle { family "SANS" style "BOLD" size 10.0 }
+						string [%(pxlabel)s]
+						fontStyle FontStyle { family "SANS" style "BOLD" size %(fontsize).1f }
 					}
 					appearance Appearance {
 						material Material { diffuseColor 1.0 0.0 0.0}
@@ -856,26 +1002,26 @@ Transform {
 				}
 			]
 		}
-		# -a axis
+		# -x axis
 		Transform {
-			translation -50.0 0.0 -50.0
+			translation -%(aboffset).1f 0.0 -50.0
 			children [
 				Shape {
-					geometry Box { size 100.0 2.0 2.0 }
+					geometry Box { size %(ab).1f %(wh).1f %(wh).1f }
 					appearance Appearance {
 						material Material { diffuseColor 0.0 1.0 0.0 }
 					}
 				}
 			]
 		}
-		# -a axis label
+		# -x axis label
 		Transform {
-			translation -108.0 -2.0 -50.0
+			translation -%(an).1f -%(wh).1f -50.0
 			children [
 				Shape {
 					geometry Text {
-						string ["-a"]
-						fontStyle FontStyle { family "SANS" style "BOLD" size 10.0 }
+						string [%(nxlabel)s]
+						fontStyle FontStyle { family "SANS" style "BOLD" size %(fontsize).1f }
 					}
 					appearance Appearance {
 						material Material { diffuseColor 0.0 1.0 0.0}
@@ -883,26 +1029,26 @@ Transform {
 				}
 			]
 		}
-		# +b axis
+		# +y axis
 		Transform {
-			translation 0.0 50.0 -50.0
+			translation 0.0 %(aboffset).1f -50.0
 			children [
 				Shape {
-					geometry Box { size 2.0 100.0 2.0 }
+					geometry Box { size %(wh).1f %(ab).1f %(wh).1f }
 					appearance Appearance {
 						material Material { diffuseColor 1.0 1.0 0.0 }
 					}
 				}
 			]
 		}
-		# +b axis label
+		# +y axis label
 		Transform {
-			translation -3.0 103.0 -50.0
+			translation -%(bp0).1f %(bp1).1f -50.0
 			children [
 				Shape {
 					geometry Text {
-						string ["+b"]
-						fontStyle FontStyle { family "SANS" style "BOLD" size 10.0 }
+						string [%(pylabel)s]
+						fontStyle FontStyle { family "SANS" style "BOLD" size %(fontsize).1f }
 					}
 					appearance Appearance {
 						material Material { diffuseColor 1.0 1.0 0.0}
@@ -910,26 +1056,26 @@ Transform {
 				}
 			]
 		}
-		# -b axis
+		# -y axis
 		Transform {
-			translation 0.0 -50.0 -50.0
+			translation 0.0 -%(aboffset).1f -50.0
 			children [
 				Shape {
-					geometry Box { size 2.0 100.0 2.0 }
+					geometry Box { size %(wh).1f %(ab).1f %(wh).1f }
 					appearance Appearance {
 						material Material { diffuseColor 0.0 0.0 1.0 }
 					}
 				}
 			]
 		}
-		# -b axis label
+		# -y axis label
 		Transform {
-			translation -3.0 -107.0 -50.0
+			translation -%(bn0).1f -%(bn1).1f -50.0
 			children [
 				Shape {
 					geometry Text {
-						string ["-b"]
-						fontStyle FontStyle { family "SANS" style "BOLD" size 10.0 }
+						string [%(nylabel)s]
+						fontStyle FontStyle { family "SANS" style "BOLD" size %(fontsize).1f }
 					}
 					appearance Appearance {
 						material Material { diffuseColor 0.0 0.0 1.0}
@@ -937,8 +1083,25 @@ Transform {
 				}
 			]
 		}
-"""
+		# Zero
+		Transform {
+			translation -%(Ln).1f -%(wh).1f -55.0
+			children [
+				Shape {
+					geometry Text {
+						string ["0"]
+						fontStyle FontStyle { family "SANS" style "BOLD" size %(fontsize).1f }
+					}
+					appearance Appearance {
+						material Material { diffuseColor 0.7 0.7 0.7}
+					}
+				}
+			]
+		}
+""" % values
 		children = []
+		sqrt3_100 = math.sqrt(3) * 100
+		sqrt3_50 = math.sqrt(3) * 50
 		for entry in data.itervalues():
 			X, Y, Z = colormath.adapt(entry["XYZ_X"],
 									  entry["XYZ_Y"],
@@ -946,13 +1109,72 @@ Transform {
 									  white,
 									  cat=cat)
 			L, a, b = colormath.XYZ2Lab(X, Y, Z)
-			if devicelocations:
+			if colorspace == "RGB":
 				# Fudge device locations into Lab space
 				x, y, z = (entry["RGB_G"] - 50,
 						   entry["RGB_B"] - 50,
 						   entry["RGB_R"] - 50)
-			else:
+			elif colorspace == "HSI":
+				H, S, z = colormath.RGB2HSI(entry["RGB_R"] / 100.0,
+											entry["RGB_G"] / 100.0,
+											entry["RGB_B"] / 100.0)
+				rad = H * 360 * math.pi / 180
+				x, y = S * z * math.cos(rad), S * z * math.sin(rad)
+				# Fudge device locations into Lab space
+				x, y, z = x * sqrt3_100, y * sqrt3_100, z * sqrt3_100 - sqrt3_50
+			elif colorspace == "HSL":
+				H, S, z = colormath.RGB2HSL(entry["RGB_R"] / 100.0,
+											entry["RGB_G"] / 100.0,
+											entry["RGB_B"] / 100.0)
+				rad = H * 360 * math.pi / 180
+				if z > .5:
+					S *= 1 - z
+				else:
+					S *= z
+				x, y = S * math.cos(rad), S * math.sin(rad)
+				# Fudge device locations into Lab space
+				x, y, z = x * sqrt3_100, y * sqrt3_100, z * sqrt3_100 - sqrt3_50
+			elif colorspace == "HSV":
+				H, S, z = colormath.RGB2HSV(entry["RGB_R"] / 100.0,
+											entry["RGB_G"] / 100.0,
+											entry["RGB_B"] / 100.0)
+				rad = H * 360 * math.pi / 180
+				x, y = S * z * math.cos(rad), S * z * math.sin(rad)
+				# Fudge device locations into Lab space
+				x, y, z = x * sqrt3_50, y * sqrt3_50, z * sqrt3_100 - sqrt3_50
+			elif colorspace == "Lab":
 				x, y, z = a, b, L - 50
+			elif colorspace in ("DIN99", "DIN99b"):
+				if colorspace == "DIN99":
+					L99, a99, b99 = colormath.Lab2DIN99(L, a, b)
+				else:
+					L99, a99, b99 = colormath.Lab2DIN99b(L, a, b)
+				x, y, z = a99, b99, L99 - 50
+			elif colorspace in ("DIN99c", "DIN99d"):
+				if colorspace == "DIN99c":
+					L99, a99, b99 = colormath.XYZ2DIN99c(X, Y, Z)
+				else:
+					L99, a99, b99 = colormath.XYZ2DIN99d(X, Y, Z)
+				x, y, z = a99, b99, L99 - 50
+			elif colorspace in ("LCH(ab)", "LCH(uv)"):
+				if colorspace == "LCH(ab)":
+					L, C, H = colormath.Lab2LCHab(L, a, b)
+				else:
+					L, u, v = colormath.XYZ2Luv(X, Y, Z)
+					L, C, H = colormath.Luv2LCHuv(L, u, v)
+				x, y, z = H - 180, C - 100, L - 50
+			elif colorspace == "Luv":
+				L, u, v = colormath.XYZ2Luv(X, Y, Z)
+				x, y, z = u, v, L - 50
+			elif colorspace == "Lu'v'":
+				L, u_, v_ = colormath.XYZ2Lu_v_(X, Y, Z)
+				x, y, z = ((u_ + offsetx) * scale,
+						   (v_ + offsety) * scale, L / 100.0 * maxz - 50)
+			elif colorspace == "xyY":
+				x, y, Y = colormath.XYZ2xyY(X, Y, Z)
+				x, y, z = ((x + offsetx) * scale,
+						   (y + offsety) * scale,
+						   Y / 100.0 * maxz - 50)
 			if RGB_black_offset != 40:
 				# Keep reference hue and saturation
 				# Lab to sRGB using reference black offset of 40 like Argyll CMS
@@ -976,14 +1198,33 @@ Transform {
 									 "B": B + .05,
 									 "radius": radius})
 		children = "".join(children)
-		vrml = vrml % {"children": children,
-					   "axes": axes}
+		# Choose viewpoint fov and z position based on colorspace
+		fov = 45
+		z = 340
+		if colorspace in ("LCH(ab)", "LCH(uv)"):
+			# Use a very narrow field of view for LCH
+			fov /= 16.0
+			z *= 16
+		elif colorspace.startswith("DIN99"):
+			fov /= scale
+		out = vrml % {"children": children,
+					  "axes": axes,
+					  "fov": fov / 180.0 * math.pi,
+					  "z": z}
+		if format != "VRML":
+			safe_print("Generating", format)
+			x3d = x3dom.vrml2x3dom(out)
+			if format == "HTML":
+				out = x3d.html(title=os.path.basename(filename))
+			else:
+				out = x3d.x3d()
 		if compress:
 			writer = GzipFileProper
 		else:
 			writer = open
-		with writer(filename, 'w') as vrmlfile:
-			vrmlfile.write(vrml)
+		safe_print("Writing", filename)
+		with writer(filename, "wb") as outfile:
+			outfile.write(out)
 	
 	@property
 	def NUMBER_OF_FIELDS(self):
@@ -1254,11 +1495,12 @@ Transform {
 			elif "CMYK_C" in data_format.values():
 				white = {"CMYK_C": 0, "CMYK_M": 0, "CMYK_Y": 0, "CMYK_K": 0}
 			else:
-				return
+				white = None
 			if white:
 				white = self.queryi1(white)
-				if not white:
-					white = self.queryv1("APPROX_WHITE_POINT")
+			if not white:
+				for key in ("LUMINANCE_XYZ_CDM2", "APPROX_WHITE_POINT"):
+					white = self.queryv1(key)
 					if white:
 						try:
 							white = [float(v) for v in white.split()]
@@ -1270,29 +1512,32 @@ Transform {
 								white = {"XYZ_X": white[0],
 										 "XYZ_Y": white[1],
 										 "XYZ_Z": white[2]}
+								break
 							else:
-								return
-				if white and (("XYZ_X" in white and
-							   "XYZ_Y" in white and
-							   "XYZ_Z" in white) or ("LAB_L" in white and
-													 "LAB_B" in white and
-													 "LAB_B" in white)):
-					if colorspace == "XYZ":
-						if "XYZ_X" in white:
-							return white["XYZ_X"], white["XYZ_Y"], white["XYZ_Z"]
-						else:
-							return colormath.Lab2XYZ(white["LAB_L"],
-													 white["LAB_A"],
-													 white["LAB_B"],
-													 scale=100)
-					elif colorspace == "Lab":
-						if "LAB_L" in white:
-							return white["LAB_L"], white["LAB_A"], white["LAB_B"]
-						else:
-							return colormath.XYZ2Lab(white["XYZ_X"],
-													 white["XYZ_Y"],
-													 white["XYZ_Z"])
-					return white
+								white = None
+				if not white:
+					return
+			if white and (("XYZ_X" in white and
+						   "XYZ_Y" in white and
+						   "XYZ_Z" in white) or ("LAB_L" in white and
+												 "LAB_B" in white and
+												 "LAB_B" in white)):
+				if colorspace == "XYZ":
+					if "XYZ_X" in white:
+						return white["XYZ_X"], white["XYZ_Y"], white["XYZ_Z"]
+					else:
+						return colormath.Lab2XYZ(white["LAB_L"],
+												 white["LAB_A"],
+												 white["LAB_B"],
+												 scale=100)
+				elif colorspace == "Lab":
+					if "LAB_L" in white:
+						return white["LAB_L"], white["LAB_A"], white["LAB_B"]
+					else:
+						return colormath.XYZ2Lab(white["XYZ_X"],
+												 white["XYZ_Y"],
+												 white["XYZ_Z"])
+				return white
 	
 	def has_cie(self):
 		"""
