@@ -116,6 +116,41 @@ def specialpow(a, b):
 	return v * signScale
 
 
+def DICOM(j, inverse=False):
+	if inverse:
+		log10Y = math.log10(j)
+		A = 71.498068
+		B = 94.593053
+		C = 41.912053
+		D = 9.8247004
+		E = 0.28175407
+		F = -1.1878455
+		G = -0.18014349
+		H = 0.14710899
+		I = -0.017046845
+		return (A + B * log10Y + C * math.pow(log10Y, 2) +
+				D * math.pow(log10Y, 3) + E * math.pow(log10Y, 4) +
+				F * math.pow(log10Y, 5) + G * math.pow(log10Y, 6) +
+				H * math.pow(log10Y, 7) + I * math.pow(log10Y, 8))
+	else:
+		logj = math.log(j)
+		a = -1.3011877
+		b = -2.5840191E-2
+		c = 8.0242636E-2
+		d = -1.0320229E-1
+		e = 1.3646699E-1
+		f = 2.8745620E-2
+		g = -2.5468404E-2
+		h = -3.1978977E-3
+		k = 1.2992634E-4
+		m = 1.3635334E-3
+		return ((a + c * logj + e * math.pow(logj, 2) +
+				 g * math.pow(logj, 3) + m * math.pow(logj, 4))
+				/
+				(1 + b * logj + d * math.pow(logj, 2) + f * math.pow(logj, 3) +
+				 h * math.pow(logj, 4) + k * math.pow(logj, 5)))
+
+
 rgb_spaces = {
 	# http://brucelindbloom.com/WorkingSpaceInfo.html
 	# ACES: https://github.com/ampas/aces-dev/blob/master/docs/ACES_1.0.1.pdf?raw=true
@@ -228,11 +263,13 @@ def apply_bpc(X, Y, Z, bp_in, bp_out, wp_out="D50", weight=False):
 	"""
 	wp_out = get_whitepoint(wp_out)
 	XYZ = [X, Y, Z]
+	bp_in = list(bp_in)
+	bp_out = list(bp_out)
 	for i, v in enumerate(XYZ):
 		if weight and v > 0:
-			XYZ[i] = ((wp_out[i] - bp_out[i]) * v - wp_out[i] * (bp_in[i] * (bp_in[i] / v) - bp_out[i])) / (wp_out[i] - bp_in[i] * (bp_in[i] / v))
-		else:
-			XYZ[i] = ((wp_out[i] - bp_out[i]) * v - wp_out[i] * (bp_in[i] - bp_out[i])) / (wp_out[i] - bp_in[i])
+			bp_in[i] *= (bp_in[i] / v)
+			bp_out[i] *= (bp_out[i] / v)
+		XYZ[i] = ((wp_out[i] - bp_out[i]) * v - wp_out[i] * (bp_in[i] - bp_out[i])) / (wp_out[i] - bp_in[i])
 	return XYZ
 
 
@@ -248,10 +285,15 @@ def interp(x, xp, fp, left=None, right=None):
 	interp(0, [0, 0], [0, 1]) will return 0
 	
 	"""
+	if not isinstance(x, (int, long, float, complex)):
+		yi = []
+		for n in x:
+			yi.append(interp(n, xp, fp, left, right))
+		return yi
 	if x in xp:
 		return fp[xp.index(x)]
-	elif x < fp[0]:
-		return xp[0] if left is None else left
+	elif x < xp[0]:
+		return fp[0] if left is None else left
 	elif x > xp[-1]:
 		return fp[-1] if right is None else right
 	else:
@@ -263,7 +305,7 @@ def interp(x, xp, fp, left=None, right=None):
 				lower = i
 			elif v > x and i < higher:
 				higher = i
-		step = x - xp[lower]
+		step = float(x - xp[lower])
 		steps = (xp[higher] - xp[lower]) / step
 		return fp[lower] + (fp[higher] - fp[lower]) / steps
 
@@ -530,6 +572,79 @@ def CIEDCCT2XYZ(T, scale=1.0):
 		return xyY2XYZ(*xyY)
 
 
+def DIN992Lab(L99, a99, b99, kCH=1.0, kE=1.0):
+	C99, H99 = DIN99familyab2DIN99CH(a99, b99)
+	return DIN99LCH2Lab(L99, C99, H99, kCH, kE)
+
+
+def DIN99b2Lab(L99, a99, b99):
+	C99, H99 = DIN99familyab2DIN99CH(a99, b99)
+	return DIN99bcdLCH2Lab(L99, C99, H99, 0, 303.67, .0039, 26, .83, 23, .075)
+
+
+def DIN99c2Lab(L99, a99, b99, whitepoint=None):
+	C99, H99 = DIN99familyab2DIN99CH(a99, b99)
+	return DIN99bcdLCH2Lab(L99, C99, H99, .1, 317.651, .0037, 0, .94, 23, .066,
+						   whitepoint)
+
+
+def DIN99d2Lab(L99, a99, b99, whitepoint=None):
+	C99, H99 = DIN99familyab2DIN99CH(a99, b99)
+	return DIN99bcdLCH2Lab(L99, C99, H99, .12, 325.221, .0036, 50, 1.14, 22.5,
+						   .06, whitepoint)
+
+
+def DIN99LCH2Lab(L99, C99, H99, kCH, kE=1.0):
+	G = (math.exp(.045 * C99 * kCH * kE) - 1) / .045
+	return DIN99familyLHCG2Lab(L99, H99, C99, G, kE, 105.51, .0158, 16, .7)
+
+
+def DIN99bcdLCH2Lab(L99, C99, H99, x, l1, l2, deg, f1, c1, c2, whitepoint=None):
+	G = (math.exp(C99 / c1) - 1) / c2
+	H99 -= deg
+	L, a, b = DIN99familyLHCG2Lab(L99, H99, C99, G, 1.0, l1, l2, deg, f1)
+	if x:
+		X, Y, Z = [v * 100 for v in Lab2XYZ(L, a, b, whitepoint)]
+		X = (X + x * Z) / (1 + x)
+		L, a, b = XYZ2Lab(X, Y, Z, whitepoint)
+	return L, a, b
+
+
+def DIN99familyLHCG2Lab(L99, H99, C99, G, kE, l1, l2, deg, f1):
+	L = (math.exp((L99 * kE) / l1) - 1) / l2
+	h99ef = H99 * math.pi / 180
+	e = G * math.cos(h99ef)
+	f = G * math.sin(h99ef)
+	rad = deg * math.pi / 180
+	a = e * math.cos(rad) - (f / f1) * math.sin(rad)
+	b = e * math.sin(rad) + (f / f1) * math.cos(rad)
+	return L, a, b
+
+def DIN99familyCH2DIN99ab(C99, H99):
+	h99ef = H99 * math.pi / 180
+	return C99 * math.cos(h99ef), C99 * math.sin(h99ef)
+
+
+def DIN99familyab2DIN99CH(a99, b99):
+	C99 = math.sqrt(math.pow(a99, 2) + math.pow(b99, 2))
+	if a99 > 0:
+		if b99 >= 0:
+			h99ef = math.atan2(b99, a99)
+		else:
+			h99ef = 2 * math.pi + math.atan2(b99, a99)
+	elif a99 < 0:
+		h99ef = math.atan2(b99, a99)
+	else:
+		if b99 > 0:
+			h99ef = math.pi / 2
+		elif b99 < 0:
+			h99ef = (3 * math.pi) / 2
+		else:
+			h99ef = 0.0
+	H99 = h99ef * 180 / math.pi
+	return C99, H99
+
+
 def HSV2RGB(H, S, V, scale=1.0):
 	if S == 0:
 		return (V * scale, ) * 3
@@ -600,6 +715,62 @@ def LCHab2Lab(L, C, H):
 	return L, a, b
 
 
+def Lab2DIN99(L, a, b, kCH=1.0, kE=1.0):
+	L99, C99, H99 = Lab2DIN99LCH(L, a, b, kCH, kE)
+	a99, b99 = DIN99familyCH2DIN99ab(C99, H99)
+	return L99, a99, b99
+
+
+def Lab2DIN99b(L, a, b, kE=1.0):
+	L99, C99, H99 = Lab2DIN99bLCH(L, a, b, kE)
+	a99, b99 = DIN99familyCH2DIN99ab(C99, H99)
+	return L99, a99, b99
+
+
+def Lab2DIN99c(L, a, b, kE=1.0, whitepoint=None):
+	X, Y, Z = [v * 100 for v in Lab2XYZ(L, a, b, whitepoint)]
+	return XYZ2DIN99c(X, Y, Z, whitepoint)
+
+
+def Lab2DIN99d(L, a, b, kE=1.0, whitepoint=None):
+	X, Y, Z = [v * 100 for v in Lab2XYZ(L, a, b, whitepoint)]
+	return XYZ2DIN99d(X, Y, Z, whitepoint)
+
+
+def Lab2DIN99LCH(L, a, b, kCH=1.0, kE=1.0):
+	L99, G, h99ef, rad = Lab2DIN99familyLGhrad(L, a, b, kE, 105.51, .0158, 16, .7)
+	C99 = math.log(1 + .045 * G) / .045 * kCH * kE
+	H99 = h99ef * 180 / math.pi
+	return L99, C99, H99
+
+
+def Lab2DIN99bLCH(L, a, b, kE=1.0):
+	return Lab2DIN99bcdLCH(L, a, b, 303.67, .0039, 26, .83, 23, .075)
+
+
+def Lab2DIN99bcdLCH(L, a, b, l1, l2, deg, f1, c1, c2):
+	L99, G, h99ef, rad = Lab2DIN99familyLGhrad(L, a, b, 1.0, l1, l2, deg, f1)
+	C99 = c1 * math.log(1 + c2 * G)
+	H99 = h99ef * 180 / math.pi + deg
+	return L99, C99, H99
+
+
+def Lab2DIN99familyLGhrad(L, a, b, kE, l1, l2, deg, f1):
+	L99 = (1.0 / kE) * l1 * math.log(1 + l2 * L)
+	rad = deg * math.pi / 180
+	if rad:
+		ar = math.cos(rad)  # a rotation term
+		br = math.sin(rad)  # b rotation term
+		e = a * ar + b * br
+		f = f1 * (b * ar - a * br)
+	else:
+		e = a
+		f = f1 * b
+	G = math.sqrt(math.pow(e, 2) + math.pow(f, 2))
+	h99ef = math.atan2(f, e)
+	return L99, G, h99ef, rad
+
+
 def Lab2LCHab(L, a, b):
 	C = math.sqrt(math.pow(a, 2) + math.pow(b, 2))
 	H = 180.0 * math.atan2(b, a) / math.pi
@@ -670,6 +841,14 @@ def Lab2xyY(L, a, b, whitepoint=None, scale=1.0):
 	return XYZ2xyY(X, Y, Z, whitepoint)
 
 
+def Luv2LCHuv(L, u, v):
+	C = math.sqrt(math.pow(u, 2) + math.pow(v, 2))
+	H = 180.0 * math.atan2(v, u) / math.pi
+	if (H < 0.0):
+		H += 360.0
+	return L, C, H
+
+
 def Luv2RGB(L, u, v, rgb_space=None, scale=1.0, round_=False, clamp=True,
 			whitepoint=None):
 	""" Convert from Luv to RGB """
@@ -707,6 +886,53 @@ def Luv2XYZ(L, u, v, whitepoint=None, scale=1.0):
 	return tuple([v * scale for v in X, Y, Z])
 
 
+def RGB2HSI(R, G, B, scale=1.0):
+	I = (R + G + B) / 3.0
+	if I:
+		S = 1 - min(R, G, B) / I
+	else:
+		S = 0
+	if not R == G == B:
+		H = math.atan2(math.sqrt(3) * (G - B), 2 * R - G - B) / math.pi / 2
+		if H < 0:
+			H += 1.0
+		if H > 1:
+			H -= 1.0
+	else:
+		H = 0
+	return H * scale, S * scale, I * scale
+
+
+def RGB2HSL(R, G, B, scale=1.0):
+	RGB_min = min(R, G, B)
+	RGB_max = max(R, G, B)
+	delta = RGB_max - RGB_min
+	L = (RGB_max + RGB_min) / 2.0
+	if delta == 0:
+		# Gray
+		H = 0
+		S = 0
+	else:
+		if L < .5:
+			S = delta / (RGB_max + RGB_min)
+		else:
+			S = delta / (2.0 - RGB_max - RGB_min)
+		delta_R = (((RGB_max - R) / 6.0) + (delta / 2.0) ) / delta
+		delta_G = (((RGB_max - G) / 6.0) + (delta / 2.0) ) / delta
+		delta_B = (((RGB_max - B) / 6.0) + (delta / 2.0) ) / delta
+		if R == RGB_max:
+			H = delta_B - delta_G
+		elif G == RGB_max:
+			H = (1.0 / 3.0) + delta_R - delta_B
+		elif B == RGB_max:
+			H = (2.0 / 3.0) + delta_G - delta_R
+		if H < 0:
+			H += 1.0
+		if H > 1:
+			H -= 1.0
+	return H * scale, S * scale, L * scale
+
+
 def RGB2HSV(R, G, B, scale=1.0):
 	RGB_min = min( R, G, B )
 	RGB_max = max( R, G, B )
@@ -1075,6 +1301,39 @@ def XYZ2CCT(X, Y, Z):
 	return p
 
 
+def XYZ2DIN99(X, Y, Z, whitepoint=None):
+	L, a, b = XYZ2Lab(X, Y, Z, whitepoint)
+	return Lab2DIN99(L, a, b)
+
+
+def XYZ2DIN99b(X, Y, Z, whitepoint=None):
+	L, a, b = XYZ2Lab(X, Y, Z, whitepoint)
+	return Lab2DIN99b(L, a, b)
+
+
+def XYZ2DIN99c(X, Y, Z, whitepoint=None):
+	return XYZ2DIN99cd(X, Y, Z, .1, 317.651, .0037, 0, .94, 23, .066,
+					   whitepoint)
+
+
+def XYZ2DIN99cd(X, Y, Z, x, l1, l2, deg, f1, c1, c2, whitepoint=None):
+	L99, C99, H99 = XYZ2DIN99cdLCH(X, Y, Z, x, l1, l2, deg, f1, c1, c2,
+								   whitepoint)
+	a99, b99 = DIN99familyCH2DIN99ab(C99, H99)
+	return L99, a99, b99
+
+
+def XYZ2DIN99cdLCH(X, Y, Z, x, l1, l2, deg, f1, c1, c2, whitepoint=None):
+	X = (1 + x) * X - x * Z
+	L, a, b = XYZ2Lab(X, Y, Z, whitepoint)
+	return Lab2DIN99bcdLCH(L, a, b, l1, l2, deg, f1, c1, c2)
+
+
+def XYZ2DIN99d(X, Y, Z, whitepoint=None):
+	return XYZ2DIN99cd(X, Y, Z, .12, 325.221, .0036, 50, 1.14, 22.5, .06,
+					   whitepoint)
+
+
 def XYZ2Lab(X, Y, Z, whitepoint=None):
 	"""
 	Convert from XYZ to Lab.
@@ -1750,6 +2009,21 @@ class gam_fits(object):
 		self.roo = roo  # 0% input target
 
 
+class Interp(object):
+
+	def __init__(self, xp, fp, left=None, right=None):
+		self.xp = xp
+		self.fp = fp
+		self.left = left
+		self.right = right
+		self.lookup = {}
+
+	def __call__(self, x):
+		if not x in self.lookup:
+			self.lookup[x] = interp(x, self.xp, self.fp, self.left, self.right)
+		return self.lookup[x]
+
+
 class BT1886(object):
 	# Adapted from ArgyllCMS xicc/xicc.c
 
