@@ -26,7 +26,8 @@ else:
 	from defaultpaths import (xdg_config_dir_default, xdg_config_home, 
 							  xdg_data_home, xdg_data_home_default, 
 							  xdg_data_dirs)
-from defaultpaths import autostart, autostart_home
+from defaultpaths import (autostart, autostart_home, iccprofiles,
+						  iccprofiles_home)
 from meta import name as appname, build, lastmod, version
 from options import ascii, debug, verbose
 from safe_print import enc, fs_enc, original_codepage
@@ -42,23 +43,26 @@ if ascii:
 
 exe = unicode(sys.executable, fs_enc)
 exedir = os.path.dirname(exe)
+exename = os.path.basename(exe)
 
 isexe = sys.platform != "darwin" and getattr(sys, "frozen", False)
 
 if isexe and os.getenv("_MEIPASS2"):
 	os.environ["_MEIPASS2"] = os.getenv("_MEIPASS2").replace("/", os.path.sep)
 
-pyfile = exe if isexe else os.path.join(os.path.dirname(__file__), 
-										appname + ".py")
+pyfile = (exe if isexe else (os.path.isfile(sys.argv[0]) and sys.argv[0]) or
+		  os.path.join(os.path.dirname(__file__), "main.py"))
 pypath = exe if isexe else os.path.abspath(unicode(pyfile, fs_enc))
-pyname, pyext = os.path.splitext(os.path.basename(pypath))
 isapp = sys.platform == "darwin" and \
 		exe.split(os.path.sep)[-3:-1] == ["Contents", "MacOS"] and \
-		os.path.isfile(os.path.join(exedir, pyname))
+		os.path.isfile(os.path.join(exedir, appname))
 if isapp:
+	pyname, pyext = os.path.splitext(exe.split(os.path.sep)[-4])
 	pydir = os.path.normpath(os.path.join(exedir, "..", "Resources"))
 else:
-	pydir = os.path.dirname(pypath)
+	pyname, pyext = os.path.splitext(os.path.basename(pypath))
+	pydir = os.path.dirname(exe if isexe
+							else os.path.abspath(unicode(__file__, fs_enc)))
 
 data_dirs = [pydir]
 
@@ -71,7 +75,6 @@ if sys.platform == "win32":
 		# Lib\site-packages\dispcalGUI can be found
 		# (e.g. Scripts\dispcalGUI-apply-profiles)
 		data_dirs += [exedir]
-	btn_width_correction = 20
 	script_ext = ".cmd"
 	scale_adjustment_factor = 1.0
 	config_sys = os.path.join(commonappdata[0], appname)
@@ -85,7 +88,6 @@ if sys.platform == "win32":
 	exe_ext = ".exe"
 	profile_ext = ".icm"
 else:
-	btn_width_correction = 10
 	if sys.platform == "darwin":
 		script_ext = ".command"
 		mac_create_app = True
@@ -108,6 +110,8 @@ else:
 		if not datahome_default in data_dirs:
 			data_dirs += [datahome_default]
 		data_dirs += [os.path.join(dir_, appname) for dir_ in xdg_data_dirs]
+		data_dirs += [os.path.join(dir_, "argyllcms") for dir_ in
+					  xdg_data_dirs]
 		data_dirs += [os.path.join(dir_, "color", "argyll") for dir_ in
 					  xdg_data_dirs]
 		del dir_
@@ -131,7 +135,9 @@ resfiles = [
 	"xrc/extra.xrc",
 	"xrc/gamap.xrc",
 	"xrc/main.xrc",
-	"xrc/mainmenu.xrc"
+	"xrc/mainmenu.xrc",
+	"xrc/report.xrc",
+	"xrc/synthicc.xrc"
 ]
 
 bitmaps = {}
@@ -171,7 +177,7 @@ def getbitmap(name):
 			bitmaps[name].SetMaskColour("black")
 		else:
 			path = None
-			if parts[-1] == appname:
+			if parts[-1].startswith(appname):
 				path = get_data_path(os.path.join(parts[-2], "apps", parts[-1]) + ".png")
 			if not path:
 				path = get_data_path(os.path.sep.join(parts) + ".png")
@@ -230,13 +236,25 @@ def get_display_name(n=None, include_geometry=False):
 	return ""
 
 
+def get_argyll_display_number(geometry):
+	""" Translate from wx display geometry to Argyll display index """
+	geometry = "%i, %i, %ix%i" % tuple(geometry)
+	for i, display in enumerate(getcfg("displays").split(os.pathsep)):
+		if display.find("@ " + geometry) > -1:
+			if debug:
+				from log import safe_print
+				safe_print("[D] Found display %s at index %i" % 
+						   (geometry, i))
+			return i
+
+
 def get_display_number(display_no):
 	""" Translate from Argyll display index to wx display index """
 	from wxaddons import wx
 	try:
 		display = getcfg("displays").split(os.pathsep)[display_no]
 	except IndexError:
-		pass
+		return 0
 	else:
 		if display in ("Web @ localhost", "Untethered", "madVR"):
 			return 0
@@ -247,9 +265,8 @@ def get_display_number(display_no):
 					from log import safe_print
 					safe_print("[D] Found display %s at index %i" % 
 							   (geometry, i))
-				display_no = i
-				break
-	return display_no
+				return i
+	return 0
 
 
 def get_display_rects():
@@ -303,6 +320,17 @@ def get_data_path(relpath, rex=None):
 	paths = []
 	for dir_ in data_dirs:
 		curpath = os.path.join(dir_, relpath)
+		if (dir_.endswith("/argyll") and (relpath + "/").startswith("ref/") and
+			not os.path.exists(curpath)):
+			# Work-around distribution-specific differences for location
+			# of Argyll reference files
+			# Fedora and Ubuntu: /usr/share/color/argyll/ref
+			# openSUSE: /usr/share/color/argyll
+			pth = relpath.split("/", 1)[-1]
+			if pth != "ref":
+				curpath = os.path.join(dir_, pth)
+			else:
+				curpath = dir_
 		if os.path.exists(curpath):
 			if os.path.isdir(curpath):
 				try:
@@ -336,7 +364,7 @@ def runtimeconfig(pyfile):
 	
 	"""
 	from log import setup_logging
-	setup_logging()
+	setup_logging(logdir)
 	if debug or verbose >= 1:
 		from log import safe_print
 	if debug:
@@ -418,6 +446,7 @@ valid_ranges = {
 }
 
 valid_values = {
+	"3d.format": ["HTML", "VRML", "X3D"],
 	"3dlut.bitdepth.input": [8, 10, 12, 14, 16],
 	"3dlut.bitdepth.output": [8, 10, 12, 14, 16],
 	"3dlut.bt1886_gamma_type": ["b", "B"],
@@ -436,7 +465,7 @@ valid_values = {
 	# for description of per-instrument supported modes
 	"measurement_mode": [None, "B", "F", "L", "R", "a", "b", "c", "e", "f", "g",
 						 "l", "m", "n", "o", "p", "r", "x", "1", "2", "3", "4", 
-						 "5", "6", "7", "8", "9"],
+						 "5", "6", "7", "8", "9", "auto"],
 	"gamap_default_intent": ["a", "r", "p", "s"],
 	"gamap_perceptual_intent": intents,
 	"gamap_saturation_intent": intents,
@@ -448,20 +477,27 @@ valid_values = {
 	"profile.install_scope": ["l", "u"],
 	"profile.quality": ["l", "m", "h", "u"],
 	"profile.quality.b2a": ["l", "m", "h", "u", "n", None],
+	"profile.b2a.smooth.size": [9, 17, 33, 45, 65],
 	"profile.type": ["g", "G", "l", "s", "S", "x", "X"],
+	"synthprofile.black_point_compensation": [0, 1],
+	"synthprofile.trc_type": ["g", "G"],
 	"tc_algo": ["", "t", "r", "R", "q", "Q", "i", "I"],  # Q = Argyll >= 1.1.0
 	"tc_vrml_use_D50": [0, 1],
+	"tc_vrml_cie_colorspace": ["DIN99", "DIN99b", "DIN99c", "DIN99d", "LCH(ab)",
+							   "LCH(uv)", "Lab", "Luv", "Lu'v'", "xyY"],
+	"tc_vrml_device_colorspace": ["HSI", "HSL", "HSV", "RGB"],
 	"trc": ["240", "709", "l", "s"],
 	"trc.type": ["g", "G"],
 	"whitepoint.colortemp.locus": ["t", "T"]
 }
 
 defaults = {
-	"3dlut.apply_bt1886_gamma_mapping": 0,
+	"3d.format": "HTML",
+	"3dlut.apply_bt1886_gamma_mapping": 1,
 	"3dlut.bitdepth.input": 10,
 	"3dlut.bitdepth.output": 10,
-	"3dlut.bt1886_gamma": 2.2,
-	"3dlut.bt1886_gamma_type": "b",
+	"3dlut.bt1886_gamma": 2.4,
+	"3dlut.bt1886_gamma_type": "B",
 	"3dlut.encoding.input": "n",
 	"3dlut.encoding.input.backup": "n",
 	"3dlut.encoding.output": "n",
@@ -535,14 +571,13 @@ defaults = {
 	"lut_viewer.show": 0,
 	"lut_viewer.show_actual_lut": 0,
 	"measurement_mode": "l",
-	"measurement_mode_unlocked": 1,
 	"measurement_mode.adaptive": 1,
 	"measurement_mode.backup": "l",
 	"measurement_mode.highres": 0,
 	"measurement_mode.projector": 0,
 	"measurement_report.apply_bt1886_gamma_mapping": 0,
-	"measurement_report.bt1886_gamma": 2.2,
-	"measurement_report.bt1886_gamma_type": "b",
+	"measurement_report.bt1886_gamma": 2.4,
+	"measurement_report.bt1886_gamma_type": "B",
 	"measurement_report.chart": "",
 	"measurement_report.chart.fields": "RGB",
 	"measurement_report.devlink_profile": "",
@@ -579,6 +614,7 @@ defaults = {
 	"position.tcgen.x": 50,
 	"position.tcgen.y": 50,
 	"profile.black_point_compensation": 1,
+	"profile.black_point_compensation.backup": 1,
 	"profile.create_gamut_views": 1,
 	"profile.install_scope": "l" if (sys.platform != "win32" and 
 									 os.geteuid() == 0) # or 
@@ -600,9 +636,13 @@ defaults = {
 	]),
 	"profile.name.expanded": u"",
 	"profile.quality": "h",
-	"profile.quality.b2a": None,
+	"profile.quality.b2a": "h",
+	"profile.b2a.smooth": 1,
+	"profile.b2a.smooth.diagpng": 1,
+	"profile.b2a.smooth.extra": 1,
+	"profile.b2a.smooth.size": 33,
 	"profile.save_path": storage, # directory
-	"profile.type": "s",
+	"profile.type": "S",
 	"profile.update": 0,
 	"profile_loader.error.show_msg": 1,
 	"profile_loader.verify_calibration": 0,
@@ -625,6 +665,10 @@ defaults = {
 	"size.synthiccframe.h": 384,
 	"skip_legacy_serial_ports": 1,
 	"sudo.preserve_environment": 1,
+	"synthprofile.black_luminance": 0.0,
+	"synthprofile.gamma": 2.4,
+	"synthprofile.luminance": 120.0,
+	"synthprofile.trc_type": "G",
 	"tc_adaption": 0.1,
 	"tc_add_ti3_relative": 1,
 	"tc_algo": "",
@@ -653,7 +697,9 @@ defaults = {
 	"tc.saturation_sweeps.custom.B": 0.0,
 	"tc_single_channel_patches": 0,
 	"tc_vrml_black_offset": 40,
-	"tc_vrml_lab": 0,
+	"tc_vrml_cie": 0,
+	"tc_vrml_cie_colorspace": "Lab",
+	"tc_vrml_device_colorspace": "RGB",
 	"tc_vrml_device": 1,
 	"tc_vrml_use_D50": 0,
 	"tc_white_patches": 4,
@@ -677,7 +723,9 @@ defaults = {
 	"whitepoint.colortemp": 5000,
 	"whitepoint.colortemp.locus": "t",
 	"whitepoint.x": 0.345741,
-	"whitepoint.y": 0.358666
+	"whitepoint.y": 0.358666,
+	"x3dom.cache": 1,
+	"x3dom.embed": 0
 }
 lcode, lenc = locale.getdefaultlocale()
 if lcode:
@@ -794,6 +842,12 @@ def getcfg(name, fallback=True):
 				else:
 					repl = "\\1"
 				value = re.sub(pattern, repl, value)
+			elif name == "measurement_mode":
+				# Map n and r measurement modes to canonical l and c
+				# - the inverse mapping happens per-instrument in
+				# Worker.add_measurement_features(). That way we can have
+				# compatibility with old and current Argyll CMS
+				value = {"n": "l", "r": "c"}.get(value, value)
 			return value
 	if hasdef and fallback:
 		value = defval
@@ -833,7 +887,7 @@ def get_current_profile(include_display_profile=False):
 		import ICCProfile as ICCP
 		try:
 			profile = ICCP.ICCProfile(path)
-		except ICCP.ICCProfileInvalidError, exception:
+		except (IOError, ICCP.ICCProfileInvalidError), exception:
 			return
 		return profile
 	elif include_display_profile:
@@ -854,6 +908,68 @@ def get_display_profile(display_no=None):
 					exception, fn=log)
 
 
+standard_profiles = []
+
+
+def get_standard_profiles(paths_only=False):
+	if not standard_profiles:
+		import ICCProfile as ICCP
+		from log import safe_print
+		# Reference profiles (Argyll + dispcalGUI)
+		ref_icc = get_data_path("ref", "\.ic[cm]$") or []
+		# Other profiles installed on the system
+		other_icc = []
+		rex = re.compile("\.ic[cm]$", re.IGNORECASE)
+		for icc_dir in set(iccprofiles + iccprofiles_home):
+			for dirpath, dirnames, basenames in os.walk(icc_dir):
+				for basename in filter(rex.search, basenames):
+					filename, ext = os.path.splitext(basename.lower())
+					if (filename.endswith("_bas") or
+						filename.endswith("_eci") or
+						filename.endswith("adobergb1998") or
+						filename.startswith("eci-rgb") or
+						filename.startswith("ecirgb") or
+						filename.startswith("ekta space") or
+						filename.startswith("ektaspace") or
+						filename.startswith("fogra") or
+						filename.startswith("gracol") or
+						filename.startswith("iso") or
+						filename.startswith("lstar-") or
+						filename.startswith("pso_") or
+						filename.startswith("prophoto") or
+						filename.startswith("psr_") or
+						filename.startswith("psrgravure") or
+						filename.startswith("snap") or
+						filename.startswith("srgb") or
+						filename.startswith("swop") or
+						filename in ("applergb",
+									 "bestrgb",
+									 "betargb",
+									 "brucergb",
+									 "ciergb",
+									 "cie-rgb",
+									 "colormatchrgb",
+									 "donrgb",
+									 "widegamutrgb")):
+						other_icc.append(os.path.join(dirpath, basename))
+		for path in ref_icc + other_icc:
+			try:
+				profile = ICCP.ICCProfile(path, load=False)
+			except EnvironmentError:
+				pass
+			except Exception, exception:
+				safe_print(exception)
+			else:
+				if (profile.version < 4 and
+					profile.profileClass != "nmcl" and
+					profile.colorSpace != "GRAY" and
+					profile.connectionColorSpace in ("Lab", "XYZ")):
+					standard_profiles.append(profile)
+	if paths_only:
+		return [profile.fileName for profile in standard_profiles]
+	return standard_profiles
+
+
 def get_total_patches(white_patches=None, black_patches=None,
 					  single_channel_patches=None, gray_patches=None,
 					  multi_steps=None, multi_bcc_steps=None,
@@ -954,7 +1070,7 @@ def is_profile(filename=None, include_display_profile=False):
 			import ICCProfile as ICCP
 			try:
 				profile = ICCP.ICCProfile(filename)
-			except ICCP.ICCProfileInvalidError:
+			except (IOError, ICCP.ICCProfileInvalidError):
 				pass
 			else:
 				return True
@@ -969,9 +1085,9 @@ def makecfgdir(which="user", worker=None):
 			try:
 				os.makedirs(confighome)
 			except Exception, exception:
-				from debughelpers import handle_error
-				handle_error(u"Warning - could not create configuration directory "
-							 "'%s': %s" % (confighome, safe_unicode(exception)))
+				from log import safe_print
+				safe_print(u"Warning - could not create configuration directory "
+						   "'%s': %s" % (confighome, safe_unicode(exception)))
 				return False
 	elif not os.path.exists(config_sys):
 		try:
@@ -988,9 +1104,9 @@ def makecfgdir(which="user", worker=None):
 				if isinstance(result, Exception):
 					raise result
 		except Exception, exception:
-			from debughelpers import handle_error
-			handle_error(u"Warning - could not create configuration directory "
-						 "'%s': %s" % (config_sys, safe_unicode(exception)))
+			from log import safe_print
+			safe_print(u"Warning - could not create configuration directory "
+					   "'%s': %s" % (config_sys, safe_unicode(exception)))
 			return False
 	return True
 
@@ -1087,9 +1203,9 @@ def writecfg(which="user", worker=None):
 			cfgfile.write("\n".join(lines))
 			cfgfile.close()
 		except Exception, exception:
-			from debughelpers import handle_error
-			handle_error(u"Warning - could not write user configuration file "
-						 "'%s': %s" % (cfgfilename, safe_unicode(exception)))
+			from log import safe_print
+			safe_print(u"Warning - could not write user configuration file "
+					   "'%s': %s" % (cfgfilename, safe_unicode(exception)))
 			return False
 	else:
 		# system-wide config - only stores essentials ie. Argyll directory
@@ -1118,9 +1234,9 @@ def writecfg(which="user", worker=None):
 				if isinstance(result, Exception):
 					raise result
 		except Exception, exception:
-			from debughelpers import handle_error
-			handle_error(u"Warning - could not write system-wide configuration file "
-						 "'%s': %s" % (cfgfilename2, safe_unicode(exception)))
+			from log import safe_print
+			safe_print(u"Warning - could not write system-wide configuration file "
+					   "'%s': %s" % (cfgfilename2, safe_unicode(exception)))
 			return False
 	return True
 
