@@ -23,12 +23,17 @@ the wrapper script in the root directory of the source tar.gz/zip
 
 """
 
+from __future__ import with_statement
+from ConfigParser import ConfigParser
 from distutils.command.install import install
 from distutils.util import change_root, get_platform
+from fnmatch import fnmatch
+import codecs
 import distutils.core
 import glob
 import os
 import platform
+import re
 import shutil
 import subprocess as sp
 import sys
@@ -41,24 +46,243 @@ from meta import (author, author_ascii, description, longdesc, domain, name,
 				  wx_minversion)
 from util_os import relpath
 
+bits = platform.architecture()[0][:2]
 pypath = os.path.abspath(__file__)
 pydir = os.path.dirname(pypath)
 basedir = os.path.dirname(pydir)
 
+config = {"data": ["tests/*.icc"],
+		  "doc": ["LICENSE.txt",
+				  "README.html",
+				  "screenshots/*.png",
+				  "theme/*.png",
+				  "theme/*.css",
+				  "theme/*.js",
+				  "theme/icons/favicon.ico",
+				  "theme/slimbox2/*.css",
+				  "theme/slimbox2/*.js"],
+		  # Excludes for .app/.exe builds
+		  # numpy.lib.utils imports pydoc, which imports Tkinter, but 
+		  # numpy.lib.utils is not even used by dispcalGUI, so omit all 
+		  # Tk stuff
+		  "excludes": {"all": ["Tkconstants", "Tkinter", "setuptools", "tcl",
+							   "test"],
+					   "darwin": [],
+					   "win32": ["win32com.client.genpy"]},
+		  "package_data": {name: ["argyll_instruments.json",
+								  "beep.wav",
+								  "camera_shutter.wav",
+								  "lang/*.json",
+								  "linear.cal",
+								  "pnp.ids",
+								  "presets/*.icc",
+								  "quirk.json",
+								  "ref/*.cie",
+								  "ref/*.gam",
+								  "ref/*.icm",
+								  "ref/*.ti1",
+								  "report/*.css",
+								  "report/*.html",
+								  "report/*.js",
+								  "test.cal",
+								  "theme/*.png",
+								  "theme/icons/10x10/*.png",
+								  "theme/icons/16x16/*.png",
+								  "theme/icons/32x32/*.png",
+								  "theme/icons/72x72/*.png",
+								  "ti1/*.ti1",
+								  "x3d-viewer/*.css",
+								  "x3d-viewer/*.html",
+								  "x3d-viewer/*.js",
+								  "xrc/*.xrc"]},
+		  "xtra_package_data": {name: {"win32": ["theme/icons/%s-uninstall.ico"
+												 % name]}}}
+
+
+def add_lib_excludes(key, excludebits):
+	for exclude in excludebits:
+		config["excludes"][key] += [name + ".lib" + exclude,
+									"lib" + exclude]
+	for exclude in ("32", "64"):
+		for pycompat in ("25", "26", "27"):
+			if (key == "win32" and
+				(pycompat == sys.version[0] + sys.version[2] or
+				 exclude == excludebits[0])):
+				continue
+			config["excludes"][key] += [name + ".lib%s.python%s" %
+										(exclude, pycompat),
+										name + ".lib%s.python%s.RealDisplaySizeMM" %
+										(exclude, pycompat)]
+
+
+add_lib_excludes("darwin", ["64" if bits == "32" else "32"])
+add_lib_excludes("win32", ["64" if bits == "32" else "32"])
+
 msiversion = ".".join((str(version_tuple[0]), 
 					   str(version_tuple[1]), 
 					   str(version_tuple[2]) + 
 					   str(version_tuple[3])))
 
+plist_dict = {"CFBundleDevelopmentRegion": "English",
+			  "CFBundleExecutable": name,
+			  "CFBundleGetInfoString": version,
+			  "CFBundleIdentifier": ".".join(reversed(domain.split("."))),
+			  "CFBundleInfoDictionaryVersion": "6.0",
+			  "CFBundleLongVersionString": version,
+			  "CFBundleName": name,
+			  "CFBundlePackageType": "APPL",
+			  "CFBundleShortVersionString": version,
+			  "CFBundleSignature": "????",
+			  "CFBundleVersion": ".".join(map(str, version_tuple)),
+			  "NSHumanReadableCopyright": u"© %s %s" % (strftime("%Y"), author)}
+
 
 class Target:
 	def __init__(self, **kwargs):
 		self.__dict__.update(kwargs)
 
 
-def setup():
-	bits = platform.architecture()[0][:2]
+def create_app_symlinks(dist_dir, scripts):
+	maincontents_rel = os.path.join(name + ".app", "Contents")
+	# Create ref, tests, ReadMe and license symlinks in directory
+	# containing the app bundle
+	for src, tgt in [("ref", "Reference"),
+					 ("tests", "Tests"),
+					 ("README.html", "README.html"),
+					 ("LICENSE.txt", "LICENSE.txt")]:
+		tgt = os.path.join(dist_dir, tgt)
+		if os.path.islink(tgt):
+			os.unlink(tgt)
+		os.symlink(os.path.join(maincontents_rel, "Resources", src), tgt)
+	# Create standalone tools app bundles by symlinking to the main bundle
+	toolscripts = filter(lambda script: script != name,
+						 [script for script, desc in scripts])
+	for script, desc in scripts:
+		if script in (name, name + "-apply-profiles"):
+			continue
+		toolname = desc.replace(name, "").strip()
+		toolapp = os.path.join(dist_dir, toolname + ".app")
+		if os.path.isdir(toolapp):
+			if raw_input('WARNING: The output directory "%s" and ALL ITS '
+                         'CONTENTS will be REMOVED! Continue? (y/n)' % toolapp).lower() == 'y':
+				print "Removing dir", toolapp
+				shutil.rmtree(toolapp)
+			else:
+				raise SystemExit('User aborted')
+		has_tool_script = os.path.exists(os.path.join(dist_dir,
+													  maincontents_rel,
+													  'MacOS', script))
+		toolcontents = os.path.join(toolapp, "Contents")
+		os.makedirs(toolcontents)
+		subdirs = ["Frameworks", "Resources"]
+		if has_tool_script:
+			# PyInstaller
+			subdirs.append("MacOS")
+		for entry in os.listdir(os.path.join(dist_dir, maincontents_rel)):
+			if entry in subdirs:
+				os.makedirs(os.path.join(toolcontents, entry))
+				for subentry in os.listdir(os.path.join(dist_dir,
+														maincontents_rel,
+														entry)):
+					src = os.path.join(dist_dir, maincontents_rel,
+									   entry, subentry)
+					tgt = os.path.join(toolcontents, entry, subentry)
+					if subentry == "main.py":
+						# py2app
+						with open(src, "rb") as main_in:
+							py = main_in.read()
+						py = py.replace("main()",
+										"main(%r)" %
+										script[len(name) + 1:])
+						with open(tgt, "wb") as main_out:
+							main_out.write(py)
+						continue
+					if subentry == name + ".icns":
+						shutil.copy(os.path.join(pydir, "theme",
+												 "icons", 
+												 "%s.icns" % script),
+									os.path.join(toolcontents, entry, 
+												 "%s.icns" % script))
+						continue
+					if subentry == script:
+						# PyInstaller
+						os.rename(src, tgt)
+					elif subentry not in toolscripts:
+						os.symlink(os.path.join("..", "..", "..",
+												maincontents_rel, entry,
+												subentry), tgt)
+			elif entry == "Info.plist":
+				with codecs.open(os.path.join(dist_dir, maincontents_rel,
+											  entry), "r", "UTF-8") as info_in:
+					infoxml = info_in.read()
+				# CFBundleName / CFBundleDisplayName
+				infoxml = re.sub("(Name</key>\s*<string>)%s" % name,
+								 lambda match: match.group(1) +
+											   toolname, infoxml)
+				# CFBundleIdentifier
+				infoxml = infoxml.replace(".%s</string>" % name,
+										  ".%s</string>" % script)
+				# CFBundleIconFile
+				infoxml = infoxml.replace("%s.icns</string>" % name,
+										  "%s.icns</string>" % script)
+				if has_tool_script:
+					# PyInstaller
+					# CFBundleExecutable
+					infoxml = re.sub("(Executable</key>\s*<string>)%s" % name,
+									 lambda match: match.group(1) +
+												   script, infoxml)
+				with codecs.open(os.path.join(toolcontents, entry), "w",
+								 "UTF-8") as info_out:
+					info_out.write(infoxml)
+			else:
+				os.symlink(os.path.join("..", "..", maincontents_rel, entry),
+						   os.path.join(toolcontents, entry))
 
+
+def get_data(tgt_dir, key, pkgname=None, subkey=None, excludes=None):
+	""" Return configured data files """
+	files = config[key]
+	src_dir = basedir
+	if pkgname:
+		files = files[pkgname]
+		src_dir = os.path.join(src_dir, pkgname)
+		if subkey:
+			if subkey in files:
+				files = files[subkey]
+			else:
+				files = []
+	data = []
+	for pth in files:
+		if not filter(lambda exclude: fnmatch(pth, exclude), excludes or []):
+			data += [(os.path.normpath(os.path.join(tgt_dir, os.path.dirname(pth))),
+					  glob.glob(os.path.join(src_dir, pth)))]
+	return data
+
+
+def get_scripts(excludes=None):
+	# It is required that each script has an accompanying .desktop file
+	scripts = []
+	desktopfiles = glob.glob(os.path.join(pydir, "..", "misc", "*.desktop"))
+	def sortbyname(a, b):
+		a, b = [os.path.splitext(v)[0] for v in (a, b)]
+		if a > b:
+			return 1
+		elif a < b:
+			return -1
+		else:
+			return 0
+	desktopfiles.sort(sortbyname)
+	for desktopfile in desktopfiles:
+		cfg = ConfigParser()
+		cfg.read(desktopfile)
+		script = cfg.get("Desktop Entry", "Exec")
+		if not filter(lambda exclude: fnmatch(script, exclude), excludes or []):
+			scripts += [(script,
+						 cfg.get("Desktop Entry", "Name").decode("UTF-8"))]
+	return scripts
+
+
+def setup():
 	print "***", os.path.abspath(sys.argv[0]), " ".join(sys.argv[1:])
 
 	bdist_bbfreeze = "bdist_bbfreeze" in sys.argv[1:]
@@ -221,58 +445,24 @@ def setup():
 	# on Mac OS X and Windows, we want data files in the package dir
 	# (package_data will be ignored when using py2exe)
 	package_data = {
-		name: [
-			"argyll_instruments.json",
-			"beep.wav",
-			"camera_shutter.wav",
-			"pnp.ids",
-			"lang/*.json",
-			"presets/*.icc",
-			"quirk.json",
-			"ref/*.cie",
-			"ref/*.gam",
-			"ref/*.icm",
-			"ref/*.ti1",
-			"report/*.css",
-			"report/*.html",
-			"report/*.js",
-			"theme/*.png",
-			"theme/icons/10x10/*.png",
-			"theme/icons/16x16/*.png",
-			"theme/icons/32x32/*.png",
-			"theme/icons/72x72/*.png",
-			"ti1/*.ti1",
-			"xrc/*.xrc",
-			"linear.cal",
-			"test.cal"
-		] if sys.platform in ("darwin", "win32") and not do_py2app and not 
-		do_py2exe else []
+		name: config["package_data"][name]
+			  if sys.platform in ("darwin", "win32") and not do_py2app and not 
+			  do_py2exe else []
 	}
 	if sdist and sys.platform in ("darwin", "win32"):
 		package_data[name] += ["theme/icons/22x22/*.png",
 							   "theme/icons/24x24/*.png",
 							   "theme/icons/48x48/*.png",
+							   "theme/icons/128x128/*.png",
 							   "theme/icons/256x256/*.png"]
 	if sys.platform == "win32" and not do_py2exe:
 		package_data[name] += ["theme/icons/*.ico"]
+	# Scripts
+	scripts = get_scripts()
 	# Doc files
-	data_files = [
-		(os.path.join(doc, "screenshots"), 
-			glob.glob(os.path.join(pydir, "..", "screenshots", "*.png"))),
-		(os.path.join(doc, "theme"), 
-			glob.glob(os.path.join(pydir, "..", "theme", "*.png"))), 
-		(os.path.join(doc, "theme"), 
-			glob.glob(os.path.join(pydir, "..", "theme", "*.css"))), 
-		(os.path.join(doc, "theme"), 
-			glob.glob(os.path.join(pydir, "..", "theme", "*.js"))), 
-		(os.path.join(doc, "theme", "slimbox2"), 
-			glob.glob(os.path.join(pydir, "..", "theme", "slimbox2", "*.css"))), 
-		(os.path.join(doc, "theme", "slimbox2"), 
-			glob.glob(os.path.join(pydir, "..", "theme", "slimbox2", "*.js"))), 
-		(os.path.join(doc, "theme", "icons"), 
-			[os.path.join(pydir, "..", "theme", "icons", "favicon.ico")]), 
-		(doc, [os.path.join(pydir, "..", "README.html")])
-	] if not is_rpm_build or doc_layout.startswith("deb") else []
+	data_files = []
+	if not is_rpm_build or doc_layout.startswith("deb"):
+		data_files += get_data(doc, "doc", excludes=["LICENSE.txt"])
 	if data_files:
 		if doc_layout.startswith("deb"):
 			data_files.append((doc, [os.path.join(pydir, "..", "dist", 
@@ -284,71 +474,20 @@ def setup():
 			data_files.append((doc, [os.path.join(pydir, "..", "LICENSE.txt")]))
 	if sys.platform not in ("darwin", "win32") or do_py2app or do_py2exe:
 		# Linux/Unix or py2app/py2exe
-		data_files += [
-			(data, [os.path.join(pydir, "argyll_instruments.json")]),
-			(data, [os.path.join(pydir, "beep.wav")]), 
-			(data, [os.path.join(pydir, "camera_shutter.wav")]), 
-			(data, [os.path.join(pydir, "pnp.ids")]), 
-			(data, [os.path.join(pydir, "quirk.json")]), 
-			(os.path.join(data, "lang"), 
-				glob.glob(os.path.join(pydir, "lang", "*.json"))), 
-			(os.path.join(data, "presets"), 
-				glob.glob(os.path.join(pydir, "presets", "*.icc"))),
-			(os.path.join(data, "ref"), 
-				glob.glob(os.path.join(pydir, "ref", "*.cie"))), 
-			(os.path.join(data, "ref"), 
-				glob.glob(os.path.join(pydir, "ref", "*.gam"))), 
-			(os.path.join(data, "ref"), 
-				glob.glob(os.path.join(pydir, "ref", "*.icm"))), 
-			(os.path.join(data, "ref"), 
-				glob.glob(os.path.join(pydir, "ref", "*.ti1"))), 
-			(os.path.join(data, "report"), 
-				glob.glob(os.path.join(pydir, "report", "*.css"))),
-			(os.path.join(data, "report"), 
-				glob.glob(os.path.join(pydir, "report", "*.html"))),
-			(os.path.join(data, "report"), 
-				glob.glob(os.path.join(pydir, "report", "*.js"))),
-			(os.path.join(data, "tests"), 
-				glob.glob(os.path.join(pydir, "..", "tests", "*.icc"))),
-			(os.path.join(data, "theme"), 
-				glob.glob(os.path.join(pydir, "theme", "*.png"))), 
-			(os.path.join(data, "ti1"), 
-				glob.glob(os.path.join(pydir, "ti1", "*.ti1"))), 
-			(os.path.join(data, "xrc"), 
-				glob.glob(os.path.join(pydir, "xrc", "*.xrc"))),
-			(data, [os.path.join(pydir, "linear.cal")]),
-			(data, [os.path.join(pydir, "test.cal")])
-		]
+		data_files += get_data(data, "package_data", name,
+							   excludes=["theme/icons/*"])
+		data_files += get_data(data, "data")
+		data_files += get_data(data, "xtra_package_data", name, sys.platform)
 		if sys.platform == "win32":
-			if do_py2exe:
-				data_files += [
-					(os.path.join(data, "theme", "icons"), 
-					 [os.path.join(pydir, "theme", "icons", name + 
-					  "-uninstall.ico")]), 
-					(os.path.join(data, "lib"), 
-					 [sys.executable, sys.executable.replace(".exe", "w.exe")]),
-					(os.path.join(data, "scripts"), 
-					 [os.path.join("scripts", name + "-3DLUT-maker")]),
-					(os.path.join(data, "scripts"), 
-					 [os.path.join("scripts", name + "-apply-profiles")]),
-					(os.path.join(data, "scripts"), 
-					 [os.path.join("scripts", name + "-curve-viewer")]),
-					(os.path.join(data, "scripts"), 
-					 [os.path.join("scripts", name + "-profile-info")]),
-					(os.path.join(data, "scripts"), 
-					 [os.path.join("scripts", name + "-synthprofile")]),
-					(os.path.join(data, "scripts"), 
-					 [os.path.join("scripts", name + "-testchart-editor")])]
-			else:
-				data_files += [(os.path.join(data, "theme", "icons"), 
-					glob.glob(os.path.join(pydir, "theme", 
-										   "icons", "*.icns|*.ico")))]
+			data_files += [(os.path.join(data, "lib"), [sys.executable])]
 		elif sys.platform != "darwin":
 			# Linux
 			data_files += [(os.path.join(os.path.dirname(data), 
 										 "applications"), 
 							[os.path.join(pydir, "..", "misc", name + 
-												".desktop")])]
+												".desktop")] +
+							glob.glob(os.path.join(pydir, "..", "misc",
+												   name + "-*.desktop")))]
 			data_files += [(autostart if os.geteuid() == 0 or prefix.startswith("/")
 							else autostart_home, 
 							[os.path.join(pydir, "..", "misc", 
@@ -401,7 +540,7 @@ def setup():
 							if os.path.isfile(fname):
 								data_files += [(entry[0], [fname])]
 		for dname in ("10x10", "16x16", "22x22", "24x24", "32x32", "48x48",
-					  "72x72", "256x256"):
+					  "72x72", "128x128", "256x256"):
 			# Only the 10x10, 16x16, 32x32 and 72x72 icons are used exclusively
 			# by the app, the other sizes of the app icon are used for the 
 			# desktop entry under Linux
@@ -409,9 +548,9 @@ def setup():
 			desktopicons = []
 			for iconpath in glob.glob(os.path.join(pydir, "theme", "icons", 
 												   dname, "*.png")):
-				if os.path.basename(iconpath) != name + ".png" or (
+				if not os.path.basename(iconpath).startswith(name) or (
 					sys.platform in ("darwin", "win32") and 
-					dname in ("10x10", "16x16", "32x32", "72x72")):
+					dname in ("16x16", "32x32")):
 					icons.append(iconpath)
 				elif sys.platform not in ("darwin", "win32"):
 					desktopicons.append(iconpath)
@@ -441,9 +580,9 @@ def setup():
 			p = sp.Popen([sys.executable, '-c', '''import os
 from distutils.core import setup, Extension
 
-setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r, 
+setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r, 
 							 define_macros=%r, extra_link_args=%r)])''' % 
-						  (name, sources, macros, link_args)] + sys.argv[1:], 
+						  (name, bits, sources, macros, link_args)] + sys.argv[1:], 
 						 stdout = sp.PIPE, stderr = sp.STDOUT)
 			lines = []
 			while True:
@@ -461,7 +600,7 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 		libraries = ["X11", "Xinerama", "Xrandr", "Xxf86vm"]
 		link_args = None
 	if sys.platform == "darwin":
-		extname = "%s.lib.RealDisplaySizeMM" % name
+		extname = "%s.lib%s.RealDisplaySizeMM" % (name, bits)
 	else:
 		extname = "%s.lib%s.python%s%s.RealDisplaySizeMM" % ((name, bits) + 
 															 sys.version_info[:2])
@@ -492,8 +631,8 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 				packages += ["%s.lib%s" % (name, tmpbits),
 							 "%s.lib%s.python%s" % (name, tmpbits, pycompat)]
 	elif sys.platform == "darwin":
-		# On Mac OS X we only want the universal binary in lib
-		pass
+		# On Mac OS X we only want the universal binaries
+		packages += ["%s.lib%s" % (name, bits)]
 	else:
 		# On Linux/Windows we want separate libraries
 		packages += ["%s.lib%s" % (name, bits),
@@ -547,7 +686,10 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 	if setuptools:
 		attrs["entry_points"] = {
 			"gui_scripts": [
-				"%s = %s.%s:main" % (name, name, name),
+				"%s = %s.main:main%s" % (script, name,
+					"" if script == name
+					else script[len(name):].lower().replace("-", "_"))
+				for script, desc in scripts
 			]
 		}
 		attrs["exclude_package_data"] = {
@@ -559,15 +701,12 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 		attrs["install_requires"] = install_requires
 		attrs["zip_safe"] = False
 	else:
-		attrs["scripts"] += [os.path.join("scripts", name)]
-		attrs["scripts"] += [os.path.join("scripts", name + "-3DLUT-maker")]
-		attrs["scripts"] += [os.path.join("scripts", name + "-curve-viewer")]
-		attrs["scripts"] += [os.path.join("scripts", name + "-profile-info")]
-		attrs["scripts"] += [os.path.join("scripts", name + "-synthprofile")]
-		attrs["scripts"] += [os.path.join("scripts", name + "-testchart-editor")]
-	
-	if sys.platform != "darwin" and (sys.platform != "win32" or not do_py2exe):
-		attrs["scripts"] += [os.path.join("scripts", name + "-apply-profiles")]
+		attrs["scripts"] += [os.path.join("scripts", script)
+							 for script, desc in
+							 filter(lambda (script, desc):
+									script != name + "-apply-profiles" or
+									sys.platform != "darwin",
+									scripts)]
 	
 	if bdist_bbfreeze:
 		attrs["setup_requires"] = ["bbfreeze"]
@@ -576,46 +715,31 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 		attrs["scripts"] += [os.path.join("util", name + "_postinstall.py")]
 		
 	if do_py2app:
-		reversedomain = domain.split(".")
-		reversedomain.reverse()
-		reversedomain = ".".join(reversedomain)
-		attrs["app"] = [os.path.join(pydir, "main.py")]
+		mainpy = os.path.join(basedir, "main.py")
+		if not os.path.exists(mainpy):
+			shutil.copy(os.path.join(basedir, "scripts", name), mainpy)
+		attrs["app"] = [mainpy]
 		dist_dir = os.path.join(pydir, "..", "dist", 
 								"py2app.%s-py%s" % (get_platform(), 
 													sys.version[:3]), 
 								name + "-" + version)
-		excludes = ["test", "Tkconstants", "Tkinter", "tcl"]
-		for excludebits in ("32", "64"):
-			excludes += ["dispcalGUI.lib%s" % excludebits]
-			for pycompat in ("25", "26", "27"):
-				excludes += ["lib%s.python%s" % (excludebits, pycompat),
-							 "lib%s.python%s.RealDisplaySizeMM" % (excludebits, pycompat)]
+		from py2app.build_app import py2app as py2app_cls
+		py2app_cls._copy_package_data = py2app_cls.copy_package_data
+		def copy_package_data(self, package, target_dir):
+			# Skip package data which is already included as data files
+			if package.identifier.split('.')[0] != name:
+				self._copy_package_data(package, target_dir)
+		py2app_cls.copy_package_data = copy_package_data
 		attrs["options"] = {
 			"py2app": {
 				"argv_emulation": True,
 				"dist_dir": dist_dir,
-				# numpy.lib.utils imports pydoc, which imports Tkinter, but 
-				# numpy.lib.utils is not even used by dispcalGUI, so omit all 
-				# Tk stuff
-				"excludes": excludes,
+				"excludes": config["excludes"]["all"] +
+							config["excludes"]["darwin"],
 				"iconfile": os.path.join(pydir, "theme", "icons", 
-										 "dispcalGUI.icns"),
+										 name + ".icns"),
 				"optimize": 0,
-				"plist": {
-					"CFBundleDevelopmentRegion": "English",
-					"CFBundleExecutable": name,
-					"CFBundleGetInfoString": version,
-					"CFBundleIdentifier": reversedomain,
-					"CFBundleInfoDictionaryVersion": "6.0",
-					"CFBundleLongVersionString": version,
-					"CFBundleName": name,
-					"CFBundlePackageType": "APPL",
-					"CFBundleShortVersionString": version,
-					"CFBundleSignature": "????",
-					"CFBundleVersion": ".".join(map(str, version_tuple)),
-					"NSHumanReadableCopyright": u"© %s %s" % (strftime("%Y"),
-															  author)
-				}
+				"plist": plist_dict
 			}
 		}
 		attrs["setup_requires"] = ["py2app"]
@@ -627,50 +751,41 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 			name + (".exe.VC90.manifest" if hasattr(sys, "version_info") and 
 			sys.version_info[:2] >= (2,6) else ".exe.manifest")))
 		attrs["windows"] = [Target(**{
-			"script": os.path.join(pydir, "..", script),
+			"script": os.path.join(basedir, "scripts", script),
 			"icon_resources": [(1, os.path.join(pydir, "theme", "icons", 
-												name + ".ico"))],
+												os.path.splitext(os.path.basename(script))[0] +
+												".ico"))],
 			"other_resources": [(24, 1, manifest_xml)],
 			"copyright": u"© %s %s" % (strftime("%Y"), author),
 			"description": desc
-		}) for desc, script in [(name, name + ".pyw"),
-								("Profile Loader",
-								 os.path.join("scripts",
-											  name + "-apply-profiles")),
-								("3DLUT Maker",
-								 os.path.join("scripts",
-											  name + "-3DLUT-maker")),
-								("Curve Viewer",
-								 os.path.join("scripts",
-											  name + "-curve-viewer")),
-								("Profile Info",
-								 os.path.join("scripts",
-											  name + "-profile-info")),
-								("Synthetic Profile Creator",
-								 os.path.join("scripts",
-											  name + "-synthprofile")),
-								("Testchart Editor",
-								 os.path.join("scripts",
-											  name + "-testchart-editor"))]]
+		}) for script, desc in scripts]
+		console_scripts = [name + "-VRML-to-X3D-converter"]
+		for console_script in console_scripts:
+			console_script_path = os.path.join(basedir, "scripts",
+											   console_script + "-console")
+			if not os.path.isfile(console_script_path):
+				shutil.copy(os.path.join(basedir, "scripts", console_script),
+							console_script_path)
+		attrs["console"] = [Target(**{
+			"script": os.path.join(basedir, "scripts", script + "-console"),
+			"icon_resources": [(1, os.path.join(pydir, "theme", "icons", 
+												os.path.splitext(os.path.basename(script))[0] +
+												".ico"))],
+			"other_resources": [(24, 1, manifest_xml)],
+			"copyright": u"© %s %s" % (strftime("%Y"), author),
+			"description": desc
+		}) for script, desc in filter(lambda (script, desc):
+									  script in console_scripts,
+									  scripts)]
 		dist_dir = os.path.join(pydir, "..", "dist", "py2exe.%s-py%s" % 
 								(get_platform(), sys.version[:3]), name + 
 								"-" + version)
-		excludes = ["Tkconstants", "Tkinter", "tcl", "dispcalGUI.lib",
-					"lib.RealDisplaySizeMM"]
-		if bits == "32":
-			excludebits = "64"
-		else:
-			excludebits = "32"
-		excludes += ["dispcalGUI.lib%s" % excludebits,
-					 "lib%s.RealDisplaySizeMM" % excludebits]
-		for pycompat in ("25", "26", "27"):
-			excludes += ["lib%s.python%s" % (excludebits, pycompat),
-						 "lib%s.python%s.RealDisplaySizeMM" % (excludebits, pycompat)]
 		attrs["options"] = {
 			"py2exe": {
 				"dist_dir": dist_dir,
 				"dll_excludes": [
 					"iertutil.dll", 
+					"MPR.dll",
 					"msvcm90.dll", 
 					"msvcp90.dll", 
 					"msvcr90.dll", 
@@ -678,10 +793,8 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 					"urlmon.dll",
 					"w9xpopen.exe"
 				],
-				# numpy.lib.utils imports pydoc, which imports Tkinter, but 
-				# numpy.lib.utils is not even used by dispcalGUI, so omit all 
-				# Tk stuff
-				"excludes": excludes, 
+				"excludes": config["excludes"]["all"] +
+							config["excludes"]["win32"], 
 				"bundle_files": 3 if wx.VERSION >= (2, 8, 10, 1) else 1,
 				"compressed": 1,
 				"optimize": 0  # 0 = don’t optimize (generate .pyc)
@@ -827,12 +940,18 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 					"tests",
 					"theme",
 					"ti1",
+					"x3d-viewer",
 					"LICENSE.txt",
 					"README.html",
 					"argyll_instruments.json",
 					"beep.wav",
 					"camera_shutter.wav",
 					name + ".desktop",
+					name + "-3DLUT-maker.desktop",
+					name + "-curve-viewer.desktop",
+					name + "-profile-info.desktop",
+					name + "-synthprofile.desktop",
+					name + "-testchart-editor.desktop",
 					"pnp.ids",
 					"quirk.json",
 					"linear.cal",
@@ -922,7 +1041,7 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 					   "edit"]
 		manifest_in += ["include LICENSE.txt", "include MANIFEST", 
 						"include MANIFEST.in", "include README.html", 
-						"include %s.pyw" % name, "include use-distutils"]
+						"include *.pyw", "include use-distutils"]
 		manifest_in += ["include " + os.path.basename(sys.argv[0])]
 		manifest_in += ["include " + 
 						os.path.splitext(os.path.basename(sys.argv[0]))[0] + 
@@ -940,6 +1059,7 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 			pkgdir = os.path.sep.join(attrs.get("package_dir", 
 												{}).get(pkg, pkg).split("/"))
 			manifest_in += ["include " + os.path.join(pkgdir, "*.py")]
+			manifest_in += ["include " + os.path.join(pkgdir, "*.pyd")]
 			manifest_in += ["include " + os.path.join(pkgdir, "*.so")]
 			for obj in attrs.get("package_data", {}).get(pkg, []):
 				manifest_in += ["include " + os.path.sep.join([pkgdir] + 
@@ -995,9 +1115,6 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 			i = sys.argv.index("py2app" if do_py2app else "py2exe")
 			if not "build_ext" in sys.argv[1:i]:
 				sys.argv.insert(i, "build_ext")
-			if len(sys.argv) < i + 2 or sys.argv[i + 1] not in ("--inplace", 
-																"-i"):
-				sys.argv.insert(i + 1, "-i")
 
 		setup(**attrs)
 		
@@ -1005,22 +1122,13 @@ setup(ext_modules=[Extension("%s.RealDisplaySizeMM", sources=%r,
 			return
 		
 		if do_py2app:
-			# Create ref, tests, ReadMe and license symlinks in directory
-			# containing the app bundle
-			os.symlink(os.path.join("dispcalGUI.app", "Contents", "Resources",
-									"ref"), os.path.join(dist_dir, "ref"))
-			os.symlink(os.path.join("dispcalGUI.app", "Contents", "Resources",
-									"tests"), os.path.join(dist_dir, "tests"))
-			os.symlink(os.path.join("dispcalGUI.app", "Contents", "Resources",
-									"README.html"), os.path.join(dist_dir,
-																 "README.html"))
-			os.symlink(os.path.join("dispcalGUI.app", "Contents", "Resources",
-									"LICENSE.txt"), os.path.join(dist_dir,
-																 "LICENSE.txt"))
+			create_app_symlinks(dist_dir, scripts)
 		
 		if do_py2exe:
-			shutil.copy(os.path.join(dist_dir, "python26.dll"),
-						os.path.join(dist_dir, "lib", "python26.dll"))
+			shutil.copy(os.path.join(dist_dir, "python%s.dll" %
+											   (sys.version[0] + sys.version[2])),
+						os.path.join(dist_dir, "lib", "python%s.dll" %
+											   (sys.version[0] + sys.version[2])))
 
 		if ((bdist_bbfreeze and sys.platform == "win32") or do_py2exe) and \
 		   sys.version_info[:2] >= (2,6):
