@@ -105,6 +105,26 @@ subst.update({u"\u00a6": u"|",
 			  u"\u2264": u"<=",
 			  u"\u2265": u"=>",})
 
+
+class StrList(list):
+	
+	""" It's a list. It's a string. It's a list of strings that behaves like a
+	string! And like a list."""
+
+	def __init__(self, seq=tuple()):
+		list.__init__(self, seq)
+
+	def __iadd__(self, text):
+		self.append(text)
+		return self
+
+	def __getattr__(self, attr):
+		return getattr(str(self), attr)
+
+	def __str__(self):
+		return "".join(self)
+
+
 def asciize(obj):
 	"""
 	Turn several unicode chars into an ASCII representation.
@@ -247,7 +267,7 @@ def safe_basestring(obj):
 	
 	"""
 	if isinstance(obj, EnvironmentError):
-		# There are three possible variations of EnvironmentError instances:
+		# There are four possible variations of EnvironmentError instances:
 		# - instance with 'errno', 'strerror', 'filename' and 'args' attributes
 		#   (created by EnvironmentError with three arguments)
 		#   NOTE: The 'args' attribute will contain only the first two arguments
@@ -255,15 +275,27 @@ def safe_basestring(obj):
 		#   (created by EnvironmentError with two arguments)
 		# - instance with just 'args' attribute
 		#   (created by EnvironmentError with one or more than three arguments)
-		args = [safe_unicode(arg) for arg in obj.args]
+		# - urllib2.URLError with empty 'args' attribute but 'reason' and
+		#   'filename' attributes
+		if hasattr(obj, "reason"):
+			if isinstance(obj.reason, basestring):
+				obj.args = (obj.reason, )
+			else:
+				obj.args = obj.reason
+		error = []
+		if obj.errno is not None:
+			error.append("[Errno %s]" % obj.errno)
+		if obj.strerror is not None:
+			if obj.filename is not None:
+				error.append(obj.strerror + ":")
+			else:
+				error.append(obj.strerror)
+		if not error:
+			error = list(obj.args)
 		if obj.filename is not None:
-			obj = "[Errno %s] %s: %r" % (args[0], args[1], obj.filename)
-		elif obj.strerror is not None:
-			obj = "[Errno %s] %s" % (args[0], args[1])
-		elif obj.errno is not None:
-			obj = "[Errno %s]" % args[0]
-		else:
-			obj = " ".join(args)
+			error.append(obj.filename)
+		error = [safe_unicode(arg) for arg in error]
+		obj = " ".join(error)
 	if not isinstance(obj, basestring):
 		try:
 			obj = unicode(obj)
