@@ -11,20 +11,16 @@ import re
 import sys
 
 from wxaddons import wx
-from wx.lib import delayedresult
 from lib.agw import labelbook
 from lib.agw.fmresources import *
 from lib.agw.pygauge import PyGauge
-from lib.agw.artmanager import ArtManager
 
-from config import (get_data_path, get_icon_bundle, getbitmap, getcfg, setcfg,
-					valid_values)
+from config import get_data_path, get_icon_bundle, getbitmap, getcfg, setcfg
 from log import get_file_logger
 from meta import name as appname
 from ordereddict import OrderedDict
 from util_str import wrap
 from wxwindows import FlatShadedButton, numpad_keycodes
-import colormath
 import config
 import localization as lang
 
@@ -169,12 +165,6 @@ class DisplayAdjustmentImageContainer(labelbook.ImageContainer):
 			if self._bCollapsed:
 				return
 
-		borderPen = wx.BLACK_PEN
-		borderPen.SetWidth(1)
-		dc.SetPen(borderPen)
-		dc.DrawLine(0, size.y, size.x, size.y)
-		dc.DrawPoint(0, size.y)
-
 		clientSize = 0
 		bUseYcoord = (style & INB_RIGHT or style & INB_LEFT)
 
@@ -911,19 +901,9 @@ class DisplayAdjustmentFrame(wx.Frame):
 		self.SetMinSize(self.GetSize())
 		
 		# Set position
-		placed = False
-		if self.GetParent():
-			if self.GetParent().IsShownOnScreen():
-				self.Center()
-				placed = True
-			else:
-				x = getcfg("position.progress.x", False) or self.GetParent().GetScreenPosition()[0]
-				y = getcfg("position.progress.y", False) or self.GetParent().GetScreenPosition()[1]
-		else:
-			x = getcfg("position.progress.x")
-			y = getcfg("position.progress.y")
-		if not placed:
-			self.SetSaneGeometry(x, y)
+		x = getcfg("position.progress.x")
+		y = getcfg("position.progress.y")
+		self.SetSaneGeometry(x, y)
 	
 	def abort(self):
 		if self.has_worker_subprocess():
