@@ -34,7 +34,8 @@ class LUT3DFrame(BaseFrame):
 		self.PostCreate(pre)
 		self.Bind(wx.EVT_CLOSE, self.OnClose)
 		
-		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16], appname))
+		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16],
+											 appname + "-3DLUT-maker"))
 		
 		self.set_child_ctrls_as_attrs(self)
 
@@ -179,7 +180,7 @@ class LUT3DFrame(BaseFrame):
 	def encoding_output_ctrl_handler(self, event):
 		encoding = self.encoding_ab[self.encoding_output_ctrl.GetSelection()]
 		if getcfg("3dlut.format") == "madVR" and encoding != "t":
-			profile = getattr(self, "output_profile")
+			profile = getattr(self, "output_profile", None)
 			if (profile and "meta" in profile.tags and
 				isinstance(profile.tags.meta, ICCP.DictType) and
 				"EDID_model" in profile.tags.meta):
@@ -394,7 +395,7 @@ class LUT3DFrame(BaseFrame):
 				return
 			try:
 				profile = ICCP.ICCProfile(path)
-			except ICCP.ICCProfileInvalidError:
+			except (IOError, ICCP.ICCProfileInvalidError):
 				if not silent:
 					show_result_dialog(Error(lang.getstr("profile.invalid")),
 									   parent=self)
@@ -456,6 +457,16 @@ class LUT3DFrame(BaseFrame):
 							self.show_encoding_controls()
 							self.enable_encoding_controls()
 							self.show_bt1886_controls()
+							if ("rTRC" in profile.tags and
+								"gTRC" in profile.tags and
+								"bTRC" in profile.tags and profile.tags.rTRC is
+								profile.tags.gTRC is profile.tags.bTRC and
+								isinstance(profile.tags.rTRC, ICCP.CurveType)):
+								# Use BT.1886 gamma mapping for SMPTE 240M /
+								# Rec. 709 TRC
+								tf = profile.tags.rTRC.get_transfer_function()
+								setcfg("3dlut.apply_bt1886_gamma_mapping",
+									  int(tf[0][1] in (-240, -709)))
 							self.apply_bt1886_cb.SetValue(bool(getcfg("3dlut.apply_bt1886_gamma_mapping")))
 							enable_bt1886_gamma = self.apply_bt1886_cb.GetValue()
 							self.bt1886_gamma_ctrl.Enable(enable_bt1886_gamma)
