@@ -1,35 +1,32 @@
 # -*- coding: utf-8 -*-
 
-from decimal import Decimal
 import math
 import os
 import re
 import subprocess as sp
 import sys
 import tempfile
-import traceback
-
-from numpy import interp
 
 from argyll_cgats import cal_to_fake_profile, vcgt_to_cal
-from config import (fs_enc, get_bitmap_as_icon, get_display_profile,
-					get_display_rects, getcfg, geticon, setcfg)
+from colormath import interp
+from config import (fs_enc, get_argyll_display_number, get_display_profile,
+					get_display_rects, getcfg, geticon, get_verified_path,
+					setcfg)
 from log import safe_print
 from meta import name as appname
 from options import debug
 from util_decimal import float2dec
 from util_os import waccess
-from worker import (Error, Worker, get_argyll_util, make_argyll_compatible_path,
-					show_result_dialog)
+from util_str import safe_unicode
+from worker import (Error, UnloggedError, Worker, get_argyll_util,
+					make_argyll_compatible_path, show_result_dialog)
 from wxaddons import FileDrop, wx
-from wxenhancedplot import _Numeric
 from wxMeasureFrame import MeasureFrame
 from wxwindows import InfoDialog
 import colormath
 import config
 import wxenhancedplot as plot
 import localization as lang
-import CGATS
 import ICCProfile as ICCP
 
 BGCOLOUR = "#333333"
@@ -142,16 +139,26 @@ class CoordinateType(list):
 			self[i][0] = vmin + colormath.specialpow(self[i][1] / 255.0, power) * (vmax - vmin)
 
 
+class PolyBox(plot.PolyLine):
+
+	def __init__(self, x, y, w, h, **attr):
+		plot.PolyLine.__init__(self, [(x, y), (x + w, y), (x + w, y + h),
+									  (x, y + h), (x, y)], **attr)
+
+
 class LUTCanvas(plot.PlotCanvas):
 
 	def __init__(self, *args, **kwargs):
 		plot.PlotCanvas.__init__(self, *args, **kwargs)
 		self.canvas.Unbind(wx.EVT_LEAVE_WINDOW)
+		self.HandCursor = wx.StockCursor(wx.CURSOR_CROSS)
+		self.GrabHandCursor = wx.StockCursor(wx.CURSOR_SIZING)
 		self.SetBackgroundColour(BGCOLOUR)
 		self.SetEnableAntiAliasing(True)
 		self.SetEnableHiRes(True)
 		self.SetEnableCenterLines(True)
 		self.SetEnableDiagonals('Bottomleft-Topright')
+		self.SetEnableDrag(True)
 		self.SetEnableGrid(False)
 		self.SetEnablePointLabel(True)
 		self.SetForegroundColour(FGCOLOUR)
@@ -160,10 +167,10 @@ class LUTCanvas(plot.PlotCanvas):
 		self.SetFontSizeTitle(FONTSIZE_LARGE)
 		self.SetGridColour(GRIDCOLOUR)
 		self.setLogScale((False,False))
-		self.SetXSpec(5)
-		self.SetYSpec(5)
 		self.SetPointLabelFunc(self.DrawPointLabel)
-		self.worker = Worker()
+		self.worker = Worker(self.Parent)
+		self.errors = []
+		self.resetzoom()
 
 	def DrawLUT(self, vcgt=None, title=None, xLabel=None, yLabel=None, 
 				r=True, g=True, b=True):
@@ -178,7 +185,6 @@ class LUTCanvas(plot.PlotCanvas):
 		Plot = plot.PolyLine
 		Plot._attributes["width"] = 1
 
-		lines = []
 		linear_points = []
 		
 		axis_y = 255.0
@@ -186,6 +192,14 @@ class LUTCanvas(plot.PlotCanvas):
 			axis_x = 100.0
 		else:
 			axis_x = 255.0
+		self.axis_x, self.axis_y = (0, axis_x), (0, axis_y)
+		if not self.last_draw:
+			self.center_x = axis_x / 2.0
+			self.center_y = axis_y / 2.0
+		self.proportional = False
+		self.spec_x = self.spec_y = 5
+
+		lines = [PolyBox(0, 0, axis_x, axis_y, colour=GRIDCOLOUR, width=1)]
 		
 		self.point_grid = [{}, {}, {}]
 
@@ -373,7 +387,7 @@ class LUTCanvas(plot.PlotCanvas):
 				legend += ['G']
 			if b:
 				legend += ['B']
-		if colour:
+		if colour and points:
 			suffix = ((', ' + lang.getstr('linear').capitalize()) if 
 						points == (linear if detect_increments else 
 									linear_points) else '')
@@ -396,13 +410,11 @@ class LUTCanvas(plot.PlotCanvas):
 										  linear_points) else '')
 				lines += [Plot(b_points, legend='B' + suffix, colour='#0080FF')]
 
-		if not lines:
-			lines += [Plot([])]
-
-		self.Draw(plot.PlotGraphics(lines, title, " ".join([xLabel,
-														    lang.getstr("in")]), 
-									" ".join([yLabel, lang.getstr("out")])), 
-				  xAxis=(0, axis_x), yAxis=(0, axis_y))
+		self._DrawCanvas(plot.PlotGraphics(lines, title,
+										   " ".join([xLabel,
+													 lang.getstr("in")]), 
+										   " ".join([yLabel,
+													 lang.getstr("out")])))
 
 	def DrawPointLabel(self, dc, mDataDict):
 		"""
@@ -417,6 +429,136 @@ class LUTCanvas(plot.PlotCanvas):
 		
 		sx, sy = mDataDict["scaledXY"]  # Scaled x, y of closest point
 		dc.DrawRectangle(sx - 3, sy - 3, 7, 7)  # 7x7 square centered on point
+        
+	def GetClosestPoints(self, pntXY, pointScaled= True):
+		"""Returns list with
+			[curveNumber, legend, index of closest point, pointXY, scaledXY, distance]
+			list for each curve.
+			Returns [] if no curves are being plotted.
+			
+			x, y in user coords
+			if pointScaled == True based on screen coords
+			if pointScaled == False based on user coords
+		"""
+		if self.last_draw == None:
+			#no graph available
+			return []
+		graphics, xAxis, yAxis= self.last_draw
+		l = []
+		for curveNum,obj in enumerate(graphics):
+			#check there are points in the curve
+			if len(obj.points) == 0 or isinstance(obj, PolyBox):
+				continue  #go to next obj
+			#[curveNumber, legend, index of closest point, pointXY, scaledXY, distance]
+			cn = [curveNum]+ [obj.getLegend()]+ obj.getClosestPoint( pntXY, pointScaled)
+			l.append(cn)
+		return l
+
+	def OnMouseDoubleClick(self, event):
+		self.resetzoom()
+		if self.last_draw:
+			self.center()
+            
+	def OnMouseLeftDown(self,event):
+		self._zoomCorner1[0], self._zoomCorner1[1]= self._getXY(event)
+		self._screenCoordinates = plot._Numeric.array(event.GetPosition())
+		if self._dragEnabled:
+			self.SetCursor(self.GrabHandCursor)
+			self.canvas.CaptureMouse()
+
+	def OnMouseLeftUp(self, event):
+		if self._dragEnabled:
+			self.SetCursor(self.HandCursor)
+			if self.canvas.HasCapture():
+				self.canvas.ReleaseMouse()
+				self._set_center()
+	
+	def _DrawCanvas(self, graphics):
+		""" Draw proportionally correct, center and zoom """
+		w = float(self.GetSize()[0] or 1)
+		h = float(self.GetSize()[1] or 1)
+		if w > 45:
+			w -= 45
+		if h > 20:
+			h -= 20
+		ratio = [w / h,
+				 h / w]
+		axis_x, axis_y = self.axis_x, self.axis_y
+		if self.proportional:
+			if ratio[0] > ratio[1]:
+				self.SetXSpec(self.spec_x * ratio[0])
+			else:
+				self.SetXSpec(self.spec_x)
+			if ratio[0] > 1:
+				axis_x=tuple([v * ratio[0] for v in axis_x])
+			if ratio[1] > ratio[0]:
+				self.SetYSpec(self.spec_y * ratio[1])
+			else:
+				self.SetYSpec(self.spec_y)
+			if ratio[1] > 1:
+				axis_y=tuple([v * ratio[1] for v in axis_y])
+		else:
+			self.SetXSpec(self.spec_x)
+			self.SetYSpec(self.spec_y)
+		x, y = self.center_x, self.center_y
+		w = (axis_x[1] - axis_x[0]) * self._zoomfactor
+		h = (axis_y[1] - axis_y[0]) * self._zoomfactor
+		axis_x = (x - w / 2, x + w / 2)
+		axis_y = (y - h / 2, y + h / 2)
+		self.Draw(graphics, axis_x, axis_y)
+	
+	def _set_center(self):
+		""" Set center position from current X and Y axis """
+		axis_x = self.GetXCurrentRange()
+		axis_y = self.GetYCurrentRange()
+		if axis_x[0] < 0:
+			if axis_x[1] < 0:
+				x = axis_x[0] + (abs(axis_x[0]) - abs(axis_x[1])) / 2.0
+			else:
+				x = axis_x[0] + (abs(axis_x[1]) + abs(axis_x[0])) / 2.0
+		else:
+			x = axis_x[0] + (abs(axis_x[1]) - abs(axis_x[0])) / 2.0
+		if axis_y[0] < 0:
+			if axis_y[1] < 0:
+				y = axis_y[0] + (abs(axis_y[0]) - abs(axis_y[1])) / 2.0
+			else:
+				y = axis_y[0] + (abs(axis_y[1]) + abs(axis_y[0])) / 2.0
+		else:
+			y = axis_y[0] + (abs(axis_y[1]) - abs(axis_y[0])) / 2.0
+		self.center_x, self.center_y = x, y
+	
+	def center(self):
+		""" Center the current graphic """
+		min_x, max_x = self.GetXMaxRange()
+		min_y, max_y = self.GetYMaxRange()
+		if self.proportional:
+			self.axis_x = self.axis_y = (min(min_x, min_y), max(max_x, max_y))
+		else:
+			self.axis_x, self.axis_y = (min_x, max_x), (min_y, max_y)
+		self.center_x = 0 + sum((min_x, max_x)) / 2
+		self.center_y = 0 + sum((min_y, max_y)) / 2
+		self.erase_pointlabel()
+		self._DrawCanvas(self.last_draw[0])
+	
+	def erase_pointlabel(self):
+		if self.GetEnablePointLabel() and self.last_PointLabel:
+			# Erase point label
+			self._drawPointLabel(self.last_PointLabel)
+			self.last_PointLabel = None
+
+	def resetzoom(self):
+		self.center_x = 0
+		self.center_y = 0
+		self._zoomfactor = 1.0
+	
+	def zoom(self, direction=1):
+		_zoomfactor = .025 * direction
+		if (self._zoomfactor + _zoomfactor > 0 and
+			self._zoomfactor + _zoomfactor <= 5):
+			self._zoomfactor += _zoomfactor
+			self._set_center()
+			self.erase_pointlabel()
+			self._DrawCanvas(self.last_draw[0])
 
 
 class LUTFrame(wx.Frame):
@@ -428,7 +570,8 @@ class LUTFrame(wx.Frame):
 		
 		wx.Frame.__init__(self, *args, **kwargs)
 		
-		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16], appname))
+		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16],
+											 appname + "-curve-viewer"))
 		
 		self.CreateStatusBar(1)
 		
@@ -450,6 +593,40 @@ class LUTFrame(wx.Frame):
 		self.box_sizer.AddGrowableCol(0)
 		self.box_sizer.AddGrowableCol(2)
 		self.box_panel.SetSizer(self.box_sizer)
+
+		self.box_sizer.Add((0, 0))
+
+		hsizer = wx.BoxSizer(wx.HORIZONTAL)
+				  
+		self.box_sizer.Add(hsizer,
+						   flag=wx.ALIGN_CENTER | wx.BOTTOM, border=8)
+		
+		self.plot_mode_select = wx.Choice(self.box_panel, -1, size=(-1, -1), 
+										  choices=[])
+		hsizer.Add(self.plot_mode_select, flag=wx.ALIGN_CENTER_VERTICAL)
+		self.Bind(wx.EVT_CHOICE, self.plot_mode_select_handler,
+				  id=self.plot_mode_select.GetId())
+
+		self.rendering_intent_select = wx.Choice(self.box_panel, -1,
+												 choices=[lang.getstr("gamap.intents.a"),
+														  lang.getstr("gamap.intents.r"),
+														  lang.getstr("gamap.intents.p"),
+														  lang.getstr("gamap.intents.s")])
+		hsizer.Add(self.rendering_intent_select, flag=wx.ALIGN_CENTER_VERTICAL | wx.LEFT,
+				   border=10)
+		self.rendering_intent_select.Bind(wx.EVT_CHOICE,
+										  self.rendering_intent_select_handler)
+		self.rendering_intent_select.SetSelection(1)
+		
+		self.direction_select = wx.Choice(self.box_panel, -1,
+										  choices=[lang.getstr("direction.backward"),
+												   lang.getstr("direction.forward.inverted")])
+		hsizer.Add(self.direction_select, flag=wx.ALIGN_CENTER_VERTICAL | wx.LEFT,
+				   border=10)
+		self.direction_select.Bind(wx.EVT_CHOICE, self.direction_select_handler)
+		self.direction_select.SetSelection(0)
+
+		self.box_sizer.Add((0, 0))
 		
 		self.box_sizer.Add((0, 0))
 		
@@ -459,19 +636,13 @@ class LUTFrame(wx.Frame):
 						   border=8)
 		
 		self.box_sizer.Add((0, 0))
-		
-		self.plot_mode_select = wx.Choice(self.box_panel, -1, size=(-1, -1), 
-										  choices=[])
-		self.plot_mode_select.SetMaxFontSize(11)
-		self.cbox_sizer.Add(self.plot_mode_select, flag=wx.ALIGN_CENTER_VERTICAL)
-		self.Bind(wx.EVT_CHOICE, self.DrawLUT, id=self.plot_mode_select.GetId())
 
 		self.reload_vcgt_btn = wx.BitmapButton(self.box_panel, -1,
 											 geticon(16, "stock_refresh"),
 											 style=wx.NO_BORDER)
 		self.reload_vcgt_btn.SetBackgroundColour(BGCOLOUR)
 		self.cbox_sizer.Add(self.reload_vcgt_btn, flag=wx.ALIGN_CENTER_VERTICAL |
-													   wx.LEFT, border=20)
+													   wx.RIGHT, border=16)
 		self.reload_vcgt_btn.Bind(wx.EVT_BUTTON, self.reload_vcgt_handler)
 		self.reload_vcgt_btn.SetToolTipString(
 			lang.getstr("calibration.load_from_display_profile"))
@@ -483,7 +654,7 @@ class LUTFrame(wx.Frame):
 											 style=wx.NO_BORDER)
 		self.apply_bpc_btn.SetBackgroundColour(BGCOLOUR)
 		self.cbox_sizer.Add(self.apply_bpc_btn, flag=wx.ALIGN_CENTER_VERTICAL |
-													 wx.LEFT, border=16)
+													 wx.RIGHT, border=16)
 		self.apply_bpc_btn.Bind(wx.EVT_BUTTON, self.apply_bpc_handler)
 		self.apply_bpc_btn.SetToolTipString(lang.getstr("black_point_compensation"))
 		self.apply_bpc_btn.SetBitmapDisabled(geticon(16, "empty"))
@@ -494,7 +665,7 @@ class LUTFrame(wx.Frame):
 												style=wx.NO_BORDER)
 		self.install_vcgt_btn.SetBackgroundColour(BGCOLOUR)
 		self.cbox_sizer.Add(self.install_vcgt_btn,
-							flag=wx.ALIGN_CENTER_VERTICAL | wx.LEFT, border=16)
+							flag=wx.ALIGN_CENTER_VERTICAL | wx.RIGHT, border=16)
 		self.install_vcgt_btn.Bind(wx.EVT_BUTTON, self.install_vcgt_handler)
 		self.install_vcgt_btn.SetToolTipString(lang.getstr("apply_cal"))
 		self.install_vcgt_btn.SetBitmapDisabled(geticon(16, "empty"))
@@ -505,15 +676,13 @@ class LUTFrame(wx.Frame):
 											 style=wx.NO_BORDER)
 		self.save_vcgt_btn.SetBackgroundColour(BGCOLOUR)
 		self.cbox_sizer.Add(self.save_vcgt_btn, flag=wx.ALIGN_CENTER_VERTICAL |
-													 wx.LEFT, border=16)
+													 wx.RIGHT, border=20)
 		self.save_vcgt_btn.Bind(wx.EVT_BUTTON, self.SaveFile)
 		self.save_vcgt_btn.SetToolTipString(lang.getstr("save_as") + " " +
 											"(*.cal)")
 		self.save_vcgt_btn.SetBitmapDisabled(geticon(16, "empty"))
 		self.save_vcgt_btn.Disable()
 		
-		self.cbox_sizer.Add((20, 0))
-		
 		self.show_as_L = wx.CheckBox(self.box_panel, -1, u"L* \u2192")
 		self.show_as_L.SetForegroundColour(FGCOLOUR)
 		self.show_as_L.SetMaxFontSize(11)
@@ -551,14 +720,33 @@ class LUTFrame(wx.Frame):
 		self.toggle_clut = wx.CheckBox(self.box_panel, -1, "LUT")
 		self.toggle_clut.SetForegroundColour(FGCOLOUR)
 		self.toggle_clut.SetMaxFontSize(11)
-		xicclu = get_argyll_util("xicclu")
-		self.toggle_clut.SetValue(bool(xicclu))
-		self.toggle_clut.Enable(bool(xicclu))
 		self.cbox_sizer.Add(self.toggle_clut, flag=wx.ALIGN_CENTER_VERTICAL |
 												   wx.LEFT, border=16)
 		self.Bind(wx.EVT_CHECKBOX, self.toggle_clut_handler,
 				  id=self.toggle_clut.GetId())
 
+		self.box_sizer.Add((0, 6))
+		self.box_sizer.Add((0, 6))
+		self.box_sizer.Add((0, 6))
+
+		self.box_sizer.Add((0, 32))
+
+		hsizer = wx.BoxSizer(wx.HORIZONTAL)
+				  
+		self.box_sizer.Add(hsizer,
+						   flag=wx.ALIGN_CENTER | wx.BOTTOM, border=8)
+
+		self.show_actual_lut_cb = wx.CheckBox(self.box_panel, -1,
+											  lang.getstr("calibration.show_actual_lut"))
+		self.show_actual_lut_cb.SetForegroundColour(FGCOLOUR)
+		self.show_actual_lut_cb.SetMaxFontSize(11)
+		hsizer.Add(self.show_actual_lut_cb, flag=wx.ALIGN_CENTER |
+						   wx.BOTTOM, border=8)
+		self.Bind(wx.EVT_CHECKBOX, self.show_actual_lut_handler,
+				  id=self.show_actual_lut_cb.GetId())
+
+		self.box_sizer.Add((0, 32))
+
 		self.save_plot_btn = wx.BitmapButton(self.box_panel, -1,
 											 geticon(16, "media-floppy"),
 											 style=wx.NO_BORDER)
@@ -571,23 +759,6 @@ class LUTFrame(wx.Frame):
 		self.save_plot_btn.SetBitmapDisabled(geticon(16, "empty"))
 		self.save_plot_btn.Disable()
 
-		self.box_sizer.Add((0, 6))
-		self.box_sizer.Add((0, 6))
-		self.box_sizer.Add((0, 6))
-
-		self.box_sizer.Add((0, 0))
-
-		self.show_actual_lut_cb = wx.CheckBox(self.box_panel, -1,
-											  lang.getstr("calibration.show_actual_lut"))
-		self.show_actual_lut_cb.SetForegroundColour(FGCOLOUR)
-		self.show_actual_lut_cb.SetMaxFontSize(11)
-		self.box_sizer.Add(self.show_actual_lut_cb,
-							flag=wx.ALIGN_CENTER | wx.BOTTOM, border=16)
-		self.Bind(wx.EVT_CHECKBOX, self.show_actual_lut_handler,
-				  id=self.show_actual_lut_cb.GetId())
-
-		self.box_sizer.Add((0, 0))
-
 		self.client.canvas.Bind(wx.EVT_MOTION, self.OnMotion)
 		
 		self.droptarget = FileDrop()
@@ -607,6 +778,14 @@ class LUTFrame(wx.Frame):
 		
 		self.Bind(wx.EVT_MOVE, self.OnMove)
 		self.Bind(wx.EVT_SIZE, self.OnSize)
+
+		children = self.GetAllChildren()
+
+		for child in children:
+			if isinstance(child, wx.Choice):
+				child.SetMaxFontSize(11)
+			child.Bind(wx.EVT_KEY_DOWN, self.key_handler)
+			child.Bind(wx.EVT_MOUSEWHEEL, self.OnWheel)
 		
 		self.display_no = -1
 		self.display_rects = get_display_rects()
@@ -616,7 +795,6 @@ class LUTFrame(wx.Frame):
 		cal.filename = self.profile.fileName or ""
 		cal.apply_bpc(True)
 		self.LoadProfile(cal_to_fake_profile(cal))
-		self.DrawLUT()
 
 	def drop_handler(self, path):
 		"""
@@ -624,25 +802,23 @@ class LUTFrame(wx.Frame):
 		
 		"""
 		filename, ext = os.path.splitext(path)
-		if ext == ".cal":
+		if ext.lower() not in (".icc", ".icm"):
 			profile = cal_to_fake_profile(path)
 			if not profile:
-				InfoDialog(self, msg=lang.getstr("calibration.file.invalid") + 
-									 "\n" + path, 
+				InfoDialog(self, msg=lang.getstr("error.file.open", path), 
 						   ok=lang.getstr("ok"), 
 						   bitmap=geticon(32, "dialog-error"))
 				return
 		else:
 			try:
 				profile = ICCP.ICCProfile(path)
-			except ICCP.ICCProfileInvalidError, exception:
+			except (IOError, ICCP.ICCProfileInvalidError), exception:
 				InfoDialog(self, msg=lang.getstr("profile.invalid") + 
 									 "\n" + path, 
 						   ok=lang.getstr("ok"), 
 						   bitmap=geticon(32, "dialog-error"))
 				return
 		self.LoadProfile(profile)
-		self.DrawLUT()
 
 	def drop_unsupported_handler(self):
 		"""
@@ -659,6 +835,14 @@ class LUTFrame(wx.Frame):
 	
 	get_display = MeasureFrame.__dict__["get_display"]
 	
+	def handle_errors(self):
+		if self.client.errors:
+			show_result_dialog(Error("\n\n".join(set(safe_unicode(error)
+													 for error in
+													 self.client.errors))),
+							   self)
+			self.client.errors = []
+	
 	def install_vcgt_handler(self, event):
 		cwd = self.worker.create_tempdir()
 		if isinstance(cwd, Exception):
@@ -667,8 +851,7 @@ class LUTFrame(wx.Frame):
 			cal = os.path.join(cwd, re.sub(r"[\\/:*?\"<>|]+",
 										   "",
 										   make_argyll_compatible_path(
-											   config.get_display_name(
-												   include_geometry=True) or 
+											   self.profile.getDescription() or 
 											   "Video LUT")))
 			vcgt_to_cal(self.profile).write(cal)
 			cmd, args = self.worker.prepare_dispwin(cal)
@@ -691,10 +874,56 @@ class LUTFrame(wx.Frame):
 													   ".cmd", ".command", 
 													   ".gam", ".gz",
 													   ".icc", ".icm",
-													   ".log",
+													   ".log", ".png",
 													   ".sh", ".ti1",
 													   ".ti3", ".wrl",
 													   ".wrz"])
+
+	def key_handler(self, event):
+		# AltDown
+		# CmdDown
+		# ControlDown
+		# GetKeyCode
+		# GetModifiers
+		# GetPosition
+		# GetRawKeyCode
+		# GetRawKeyFlags
+		# GetUniChar
+		# GetUnicodeKey
+		# GetX
+		# GetY
+		# HasModifiers
+		# KeyCode
+		# MetaDown
+		# Modifiers
+		# Position
+		# RawKeyCode
+		# RawKeyFlags
+		# ShiftDown
+		# UnicodeKey
+		# X
+		# Y
+		key = event.GetKeyCode()
+		if (event.ControlDown() or event.CmdDown()): # CTRL (Linux/Mac/Windows) / CMD (Mac)
+			if key == 83 and self.profile: # S
+				self.SaveFile()
+				return
+			else:
+				event.Skip()
+		elif key in (43, wx.WXK_NUMPAD_ADD):
+			# + key zoom in
+			self.client.zoom(-1)
+		elif key in (45, wx.WXK_NUMPAD_SUBTRACT):
+			# - key zoom out
+			self.client.zoom(1)
+		else:
+			event.Skip()
+	
+	def direction_select_handler(self, event):
+		self.toggle_clut_handler(event)
+
+	def rendering_intent_select_handler(self, event):
+		self.toggle_clut_handler(event)
 	
 	def toggle_clut_handler(self, event):
 		try:
@@ -704,6 +933,7 @@ class LUTFrame(wx.Frame):
 		else:
 			self.trc = None
 			self.DrawLUT()
+			self.handle_errors()
 
 	def show_actual_lut_handler(self, event):
 		setcfg("lut_viewer.show_actual_lut", 
@@ -750,7 +980,7 @@ class LUTFrame(wx.Frame):
 													   ".cmd", ".command", 
 													   ".gam", ".gz",
 													   ".icc", ".icm",
-													   ".log",
+													   ".log", ".png",
 													   ".sh", ".ti1",
 													   ".ti3", ".wrl",
 													   ".wrz"])
@@ -760,10 +990,8 @@ class LUTFrame(wx.Frame):
 			   self.profile.fileName != profile.fileName or \
 			   not self.profile.isSame(profile):
 				self.LoadProfile(profile)
-				self.DrawLUT()
 		else:
 			self.LoadProfile(None)
-			self.DrawLUT()
 	
 	def lookup_tone_response_curves(self, intent="r"):
 		""" Lookup Y -> RGB tone values through TRC tags or LUT """
@@ -771,7 +999,14 @@ class LUTFrame(wx.Frame):
 		mult = 2
 		size = 256 * mult  # Final number of coordinates
 
-		if (intent == "r" and (not "B2A0" in self.profile.tags or
+		if hasattr(self, "rendering_intent_select"):
+			intent = {0: "a",
+					  1: "r",
+					  2: "p",
+					  3: "s"}.get(self.rendering_intent_select.GetSelection())
+
+		if (intent == "r" and (not ("B2A0" in self.profile.tags or
+									"A2B0" in self.profile.tags) or
 						       not self.toggle_clut.GetValue()) and
 			isinstance(self.profile.tags.get("rTRC"), ICCP.CurveType) and
 			isinstance(self.profile.tags.get("gTRC"), ICCP.CurveType) and
@@ -819,130 +1054,101 @@ class LUTFrame(wx.Frame):
 			return
 
 		profile = self.profile
-		channels = {'XYZ': 3,
-					'Lab': 3,
-					'Luv': 3,
-					'YCbr': 3,
-					'Yxy': 3,
-					'RGB': 3,
-					'GRAY': 1,
-					'HSV': 3,
-					'HLS': 3,
-					'CMYK': 4,
-					'CMY': 3,
-					'2CLR': 2,
-					'3CLR': 3,
-					'4CLR': 4,
-					'5CLR': 5,
-					'6CLR': 6,
-					'7CLR': 7,
-					'8CLR': 8,
-					'9CLR': 9,
-					'ACLR': 10,
-					'BCLR': 11,
-					'CCLR': 12,
-					'DCLR': 13,
-					'ECLR': 14,
-					'FCLR': 15}.get(profile.colorSpace)
 
-		if not channels:
-			raise Error(lang.getstr("profile.unsupported",
-									(profile.profileClass,
-									 profile.colorSpace)))
-		
-		# Setup xicclu
-		xicclu = get_argyll_util("xicclu")
-		if not xicclu:
+		if profile.version >= 4:
+			self.client.errors.append(Error("\n".join([lang.getstr("profile.iccv4.unsupported"),
+													   profile.getDescription()])))
 			return
-		xicclu = xicclu.encode(fs_enc)
-		cwd = self.client.worker.create_tempdir()
-		if isinstance(cwd, Exception):
-			raise cwd
-		if sys.platform == "win32":
-			startupinfo = sp.STARTUPINFO()
-			startupinfo.dwFlags |= sp.STARTF_USESHOWWINDOW
-			startupinfo.wShowWindow = sp.SW_HIDE
-		else:
-			startupinfo = None
 
-		# Prepare profile
-		profile.write(os.path.join(cwd, "profile.icc"))
+		if (profile.colorSpace != "RGB" or
+			profile.connectionColorSpace not in ("Lab", "XYZ")):
+			if profile.colorSpace != "RGB":
+				unsupported_colorspace = profile.colorSpace
+			else:
+				unsupported_colorspace = profile.connectionColorSpace
+			self.client.errors.append(Error(lang.getstr("profile.unsupported",
+														(profile.profileClass,
+														 unsupported_colorspace))))
+			return
+		
+		if "B2A0" in profile.tags:
+			direction = {0: "b",
+						 1: "if",
+						 2: "f",
+						 3: "ib"}.get(self.direction_select.GetSelection())
+		else:
+			direction = "if"
 		
 		# Prepare input Lab values
 		XYZ_triplets = []
 		Lab_triplets = []
+		RGB_triplets = []
 		for i in xrange(0, size):
-			if intent == "a":
-				# Experimental - basically this makes the resulting
-				# response match relative colorimetric
-				X, Y, Z = colormath.Lab2XYZ(i * (100.0 / (size - 1)), 0, 0)
-				L, a, b = colormath.XYZ2Lab(*[v * 100 for v in
-											  colormath.adapt(X, Y, Z,
-															  whitepoint_destination=profile.tags.wtpt.values())])
-			else:
+			if direction in ("b", "if"):
+				##if intent == "a":
+					### Experimental - basically this makes the resulting
+					### response match relative colorimetric
+					##X, Y, Z = colormath.Lab2XYZ(i * (100.0 / (size - 1)), 0, 0)
+					##L, a, b = colormath.XYZ2Lab(*[v * 100 for v in
+												  ##colormath.adapt(X, Y, Z,
+																  ##whitepoint_destination=profile.tags.wtpt.values())])
+				##else:
 				a = b = 0
-			Lab_triplets.append(" ".join([str(i * (100.0 / (size - 1))),
-										  str(a), str(b)]))
+				Lab_triplets.append([i * (100.0 / (size - 1)), a, b])
+			else:
+				RGB_triplets.append([i * (1.0 / (size - 1))] * 3)
+		if direction in ("b", "if"):
+			idata = Lab_triplets
+		else:
+			idata = RGB_triplets
 		
-		# Lookup Lab -> RGB values through 'input' profile using xicclu
-		stderr = tempfile.SpooledTemporaryFile()
-		p = sp.Popen([xicclu, "-fb", "-i" + intent, "-pl", "profile.icc"], 
-					 stdin=sp.PIPE, stdout=sp.PIPE, stderr=stderr, 
-					 cwd=cwd.encode(fs_enc), startupinfo=startupinfo)
-		self.client.worker.subprocess = p
-		if p.poll() not in (0, None):
-			stderr.seek(0)
-			raise Error(stderr.read().strip())
+		order = {True: "n",
+				 False: "r"}.get(("B2A0" in self.profile.tags or
+								  "A2B0" in self.profile.tags) and
+								 self.toggle_clut.GetValue())
+
+		# Lookup values through 'input' profile using xicclu
 		try:
-			odata = p.communicate("\n".join(Lab_triplets))[0].splitlines()
-		except IOError:
-			stderr.seek(0)
-			raise Error(stderr.read().strip())
-		if p.wait() != 0:
-			raise IOError(''.join(odata))
-		stderr.close()
+			odata = self.worker.xicclu(profile, idata, intent,
+									   direction, order, "l")
+		except Exception, exception:
+			self.client.errors.append(Error(safe_unicode(exception)))
 
 		# Remove temporary files
 		self.client.worker.wrapup(False)
-
-		rgb_triplets = []
-		for i, line in enumerate(odata):
-			line = "".join(line.strip().split("->")).split()
-			rgb_triplet = [float(n) for n in line[channels + 2:channels + 5]]
-			rgb_triplets.append(rgb_triplet)
 		
+		if self.client.errors:
+			return
+
+		if direction in ("b", "if"):
+			RGB_triplets = odata
+		else:
+			Lab_triplets = odata
+
 		self.rTRC = CoordinateType()
 		self.gTRC = CoordinateType()
 		self.bTRC = CoordinateType()
-		for j, rgb in enumerate(rgb_triplets):
-			for i, v in enumerate(rgb):
+		for j, RGB in enumerate(RGB_triplets):
+			for i, v in enumerate(RGB):
+				v = min(v, 1.0)
 				v *= 255
-				y = colormath.Lab2XYZ(float(Lab_triplets[j].split()[0]), 0, 0)[1] * 100
+				X, Y, Z = colormath.Lab2XYZ(*Lab_triplets[j], scale=100)
+				if direction in ("b", "if"):
+					X = Z = Y
+				elif intent == "a":
+					wp = profile.tags.wtpt.ir.values()
+					X, Y, Z = colormath.adapt(X, Y, Z, wp, (1, 1, 1))
 				if i == 0:
-					self.rTRC.append([y, v])
+					self.rTRC.append([X, v])
 				elif i == 1:
-					self.gTRC.append([y, v])
+					self.gTRC.append([Y, v])
 				elif i == 2:
-					self.bTRC.append([y, v])
-
-	def move_handler(self, event):
-		if not self.IsShownOnScreen():
-			return
-		display_no, geometry, client_area = self.get_display()
-		if display_no != self.display_no:
-			self.display_no = display_no
-			# Translate from wx display index to Argyll display index
-			geometry = "%i, %i, %ix%i" % tuple(geometry)
-			for i, display in enumerate(getcfg("displays").split(os.pathsep)):
-				if display.find("@ " + geometry) > -1:
-					if debug:
-						safe_print("[D] Found display %s at index %i" % 
-								   (geometry, i))
-					# Save Argyll display index to configuration
-					setcfg("display.number", i + 1)
-					self.load_lut(get_display_profile(i))
-					break
-		event.Skip()
+					self.bTRC.append([Z, v])
+	
+	def plot_mode_select_handler(self, event):
+		self.client.resetzoom()
+		self.DrawLUT()
+		wx.CallAfter(self.client.center)
 	
 	def reload_vcgt_handler(self, event):
 		cmd, args = self.worker.prepare_dispwin(True)
@@ -954,7 +1160,7 @@ class LUTFrame(wx.Frame):
 			if isinstance(result, Exception):
 				show_result_dialog(result, self)
 			elif not result:
-				show_result_dialog(Error("".join(self.worker.errors)),
+				show_result_dialog(UnloggedError("".join(self.worker.errors)),
 								   self)
 			else:
 				self.load_lut(get_display_profile())
@@ -967,7 +1173,12 @@ class LUTFrame(wx.Frame):
 			profile.size = len(profile.data)
 			profile.is_loaded = True
 		elif not isinstance(profile, ICCP.ICCProfile):
-			profile = ICCP.ICCProfile(profile)
+			try:
+				profile = ICCP.ICCProfile(profile)
+			except (IOError, ICCP.ICCProfileInvalidError), exception:
+				show_result_dialog(Error(lang.getstr("profile.invalid") + 
+									     "\n" + profile), self)
+				return
 		self.profile = profile
 		self.rTRC = profile.tags.get("rTRC")
 		self.gTRC = profile.tags.get("gTRC")
@@ -975,10 +1186,12 @@ class LUTFrame(wx.Frame):
 		self.trc = None
 		curves = []
 		curves.append(lang.getstr('vcgt'))
-		if ((isinstance(self.rTRC, ICCP.CurveType) and
-			 isinstance(self.gTRC, ICCP.CurveType) and
-			 isinstance(self.bTRC, ICCP.CurveType)) or
-			("B2A0" in profile.tags and profile.colorSpace == "RGB")):
+		self.client.errors = []
+		self.toggle_clut.SetValue("B2A0" in profile.tags or
+								  "A2B0" in profile.tags)
+		if ((self.rTRC and self.gTRC and self.bTRC) or
+			(self.toggle_clut.GetValue() and
+			 profile.colorSpace == "RGB")):
 			try:
 				self.lookup_tone_response_curves()
 			except Exception, exception:
@@ -993,6 +1206,8 @@ class LUTFrame(wx.Frame):
 		self.plot_mode_select.SetSelection(selection)
 		self.cbox_sizer.Layout()
 		self.box_sizer.Layout()
+		self.DrawLUT()
+		self.handle_errors()
 
 	def add_tone_values(self, legend):
 		if not self.profile:
@@ -1110,20 +1325,11 @@ class LUTFrame(wx.Frame):
 		self.Freeze()
 		curves = None
 		if self.profile:
-			yLabel = []
-			if self.toggle_red.GetValue():
-				yLabel.append("R")
-			if self.toggle_green.GetValue():
-				yLabel.append("G")
-			if self.toggle_blue.GetValue():
-				yLabel.append("B")
 			if self.plot_mode_select.GetSelection() == 0:
 				if 'vcgt' in self.profile.tags:
 					curves = self.profile.tags['vcgt']
 				else:
 					curves = None
-				self.xLabel = "".join(yLabel)
-				self.yLabel = "".join(yLabel)
 			elif (self.plot_mode_select.GetSelection() == 1 and
 				  isinstance(self.rTRC, (ICCP.CurveType, CoordinateType)) and
 				  isinstance(self.gTRC, (ICCP.CurveType, CoordinateType)) and
@@ -1152,21 +1358,34 @@ class LUTFrame(wx.Frame):
 						'entryCount': len(self.rTRC),
 						'entrySize': 2
 					}
-				if self.show_as_L.GetValue():
-					self.xLabel = "L*"
-				else:
-					self.xLabel = "Y"
-				self.yLabel = "".join(yLabel)
+		yLabel = []
+		if self.toggle_red.GetValue():
+			yLabel.append("R")
+		if self.toggle_green.GetValue():
+			yLabel.append("G")
+		if self.toggle_blue.GetValue():
+			yLabel.append("B")
+		if self.plot_mode_select.GetSelection() == 0:
+			self.xLabel = "".join(yLabel)
+		else:
+			if self.show_as_L.GetValue():
+				self.xLabel = "L*"
+			else:
+				self.xLabel = "Y"
+		self.yLabel = "".join(yLabel)
+				
 		self.toggle_red.Enable(bool(curves))
 		self.toggle_green.Enable(bool(curves))
 		self.toggle_blue.Enable(bool(curves))
 		self.show_as_L.Enable(bool(curves))
 		self.show_as_L.Show(self.plot_mode_select.GetSelection() != 0)
 		self.toggle_clut.Show(self.plot_mode_select.GetSelection() == 1 and
-							  "B2A0" in self.profile.tags and
-							  isinstance(self.profile.tags.get("rTRC"), ICCP.CurveType) and
-							  isinstance(self.profile.tags.get("gTRC"), ICCP.CurveType) and
-							  isinstance(self.profile.tags.get("bTRC"), ICCP.CurveType))
+							  ("B2A0" in self.profile.tags or
+							   "A2B0" in self.profile.tags))
+		self.toggle_clut.Enable(self.plot_mode_select.GetSelection() == 1 and
+								isinstance(self.profile.tags.get("rTRC"), ICCP.CurveType) and
+								isinstance(self.profile.tags.get("gTRC"), ICCP.CurveType) and
+								isinstance(self.profile.tags.get("bTRC"), ICCP.CurveType))
 		self.save_plot_btn.Enable(bool(curves))
 		if hasattr(self, "reload_vcgt_btn"):
 			self.reload_vcgt_btn.Enable(not(self.plot_mode_select.GetSelection()) and
@@ -1187,9 +1406,16 @@ class LUTFrame(wx.Frame):
 			self.save_vcgt_btn.Enable(not(self.plot_mode_select.GetSelection()) and
 									  bool(self.profile) and "vcgt" in self.profile.tags)
 			self.save_vcgt_btn.Show(not(self.plot_mode_select.GetSelection()))
-		self.show_as_L.GetContainingSizer().Layout()
 		if hasattr(self, "show_actual_lut_cb"):
 			self.show_actual_lut_cb.Show(self.plot_mode_select.GetSelection() == 0)
+		if hasattr(self, "rendering_intent_select"):
+			self.rendering_intent_select.Show(self.plot_mode_select.GetSelection() == 1)
+		if hasattr(self, "direction_select"):
+			self.direction_select.Show(self.toggle_clut.IsShown() and
+									   self.toggle_clut.GetValue() and
+									   "B2A0" in self.profile.tags and
+									   "A2B0" in self.profile.tags)
+		self.show_as_L.GetContainingSizer().Layout()
 		if hasattr(self, "cbox_sizer"):
 			self.cbox_sizer.Layout()
 		if hasattr(self, "box_sizer"):
@@ -1200,16 +1426,17 @@ class LUTFrame(wx.Frame):
 			self.client.last_PointLabel = None
 		else:
 			pointXY = (127.5, 127.5)
-		self.client.DrawLUT(curves, title=self.profile.getDescription() if 
-										  self.profile else None, 
-							xLabel=self.xLabel,
-							yLabel=self.yLabel,
-							r=self.toggle_red.GetValue() if 
-							  hasattr(self, "toggle_red") else False, 
-							g=self.toggle_green.GetValue() if 
-							  hasattr(self, "toggle_green") else False, 
-							b=self.toggle_blue.GetValue() if 
-							  hasattr(self, "toggle_blue") else False)
+		wx.CallAfter(self.client.DrawLUT, curves,
+					 title=self.profile.getDescription() if 
+					 self.profile else None, 
+					 xLabel=self.xLabel,
+					 yLabel=self.yLabel,
+					 r=self.toggle_red.GetValue() if 
+					   hasattr(self, "toggle_red") else False, 
+					 g=self.toggle_green.GetValue() if 
+					   hasattr(self, "toggle_green") else False, 
+					 b=self.toggle_blue.GetValue() if 
+					   hasattr(self, "toggle_blue") else False)
 		self.Thaw()
 		wx.CallLater(125, self.UpdatePointLabel, pointXY)
 
@@ -1224,8 +1451,11 @@ class LUTFrame(wx.Frame):
 			xy = self.client._getXY(event)
 		else:
 			xy = event
-		self.UpdatePointLabel(xy)
 		if isinstance(event, wx.MouseEvent):
+			if not event.LeftIsDown():
+				self.UpdatePointLabel(xy)
+			else:
+				self.client.erase_pointlabel()
 			event.Skip() # Go to next handler
 
 	def OnMove(self, event=None):
@@ -1245,6 +1475,15 @@ class LUTFrame(wx.Frame):
 			setcfg("size.lut_viewer.h", h)
 		if event:
 			event.Skip()
+	
+	def OnWheel(self, event):
+		xy = wx.GetMousePosition()
+		if self.client.last_draw:
+			if event.WheelRotation < 0:
+				direction = 1.0
+			else:
+				direction = -1.0
+			self.client.zoom(direction)
 
 	def SaveFile(self, event=None):
 		"""Saves the file to the type specified in the extension. If no file
@@ -1263,6 +1502,7 @@ class LUTFrame(wx.Frame):
 		if (event and hasattr(self, "save_vcgt_btn") and
 			event.GetId() == self.save_vcgt_btn.GetId()):
 			extensions = {"cal": 1}
+			defType = "cal"
 		else:
 			extensions = {
 				"bmp": wx.BITMAP_TYPE_BMP,       # Save a Windows bitmap file.
@@ -1271,8 +1511,11 @@ class LUTFrame(wx.Frame):
 				"jpg": wx.BITMAP_TYPE_JPEG,      # Save a JPG file.
 				"png": wx.BITMAP_TYPE_PNG,       # Save a PNG file.
 				}
+			defType = "png"
 
-		fType = fileName[-3:].lower()
+		fileName += "." + defType
+
+		fType = None
 		dlg1 = None
 		while fType not in extensions:
 
@@ -1284,11 +1527,13 @@ class LUTFrame(wx.Frame):
 			else:                      # FileDialog doesn't exist: just check one
 				dlg1 = wx.FileDialog(
 					self, 
-					lang.getstr("save_as"), ".", fileName,
+					lang.getstr("save_as"),
+					get_verified_path("last_filedialog_path")[0], fileName,
 					"|".join(["%s (*.%s)|*.%s" % (ext.upper(), ext, ext)
-							  for ext in extensions.keys()]),
+							  for ext in sorted(extensions.keys())]),
 					wx.SAVE|wx.OVERWRITE_PROMPT
 					)
+				dlg1.SetFilterIndex(sorted(extensions.keys()).index(defType))
 
 			if dlg1.ShowModal() == wx.ID_OK:
 				fileName = dlg1.GetPath()
@@ -1297,6 +1542,7 @@ class LUTFrame(wx.Frame):
 														 fileName)), self)
 					return
 				fType = fileName[-3:].lower()
+				setcfg("last_filedialog_path", fileName)
 			else:                      # exit without saving
 				dlg1.Destroy()
 				return False
@@ -1318,7 +1564,7 @@ class LUTFrame(wx.Frame):
 			# Show closest point (when enbled)
 			# Make up dict with info for the point label
 			dlst = self.client.GetClosestPoint(xy, pointScaled=True)
-			if dlst != []:
+			if dlst != [] and hasattr(self.client, "point_grid"):
 				curveNum, legend, pIndex, pointXY, scaledXY, distance = dlst
 				legend = legend.split(", ")
 				R, G, B = (self.client.point_grid[0].get(pointXY[0],
@@ -1408,14 +1654,17 @@ def main(profile=None):
 	app = LUTViewer(0)
 	app.frame.worker.enumerate_displays_and_ports(check_lut_access=False,
 												  enumerate_ports=False)
-	app.frame.Bind(wx.EVT_MOVE, app.frame.move_handler, app.frame)
-	app.frame.Show(True)
 	app.frame.update_controls()
+	if profile and profile.startswith("-"):
+		profile = None
+	display_no = get_argyll_display_number(app.frame.get_display()[1])
+	setcfg("display.number", display_no + 1)
 	if profile:
 		app.frame.drop_handler(profile)
 	else:
-		app.frame.load_lut(get_display_profile())
+		app.frame.load_lut(get_display_profile(display_no))
+	app.frame.Show()
 	app.MainLoop()
 
 if __name__ == '__main__':
-    main(*sys.argv[1:2])
\ No newline at end of file
+    main(*sys.argv[max(len(sys.argv) - 1, 1):])
\ No newline at end of file
