@@ -6,8 +6,8 @@ import sys
 
 import config
 import localization as lang
-from config import (btn_width_correction, defaults, getcfg, geticon, 
-					get_bitmap_as_icon, get_data_path, get_display_number, 
+from config import (defaults, getcfg, geticon, 
+					get_argyll_display_number, get_display_number, 
 					get_display_rects, scale_adjustment_factor, setcfg,
 					writecfg)
 from debughelpers import handle_error
@@ -16,7 +16,7 @@ from meta import name as appname
 from options import debug
 from util_list import floatlist, strlist
 from util_str import safe_unicode
-from wxaddons import CustomEvent, wx
+from wxaddons import wx
 from wxwindows import ConfirmDialog, InfoDialog, InvincibleFrame
 try:
 	import RealDisplaySizeMM as RDSMM
@@ -39,7 +39,8 @@ def get_default_size():
 	display_sizes = []
 	display_sizes_mm = []
 	for display_no in xrange(len(getcfg("displays").split(os.pathsep))):
-		display_size = wx.Display(get_display_number(display_no)).Geometry[2:]
+		display_no = get_display_number(display_no)
+		display_size = wx.Display(display_no).Geometry[2:]
 		display_size_mm = []
 		if RDSMM:
 			try:
@@ -127,7 +128,6 @@ class MeasureFrame(InvincibleFrame):
 		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16], appname))
 		self.Bind(wx.EVT_CLOSE, self.close_handler, self)
 		self.Bind(wx.EVT_MOVE, self.move_handler, self)
-		self.Bind(wx.EVT_SIZE, self.size_handler, self)
 		self.panel = wx.Panel(self, -1)
 		self.sizer = wx.GridSizer(3, 1)
 		self.panel.SetSizer(self.sizer)
@@ -200,14 +200,7 @@ class MeasureFrame(InvincibleFrame):
 												 wx.ALIGN_CENTER_HORIZONTAL | 
 												 wx.ALL, border=10)
 		self.measurebutton.SetMaxFontSize(11)
-		self.measurebutton.SetInitialSize((self.measurebutton.GetSize()[0] + 
-										   btn_width_correction, -1))
 
-		min_size = max(self.sizer.GetMinSize())
-		# Make sure the min size is quadratic and large enough to accomodate 
-		# all controls
-		self.SetMinSize((min_size, min_size))
-		self.SetMaxSize((20000, 20000))
 		self.display_no = wx.Display.GetFromWindow(self)
 		self.display_rects = get_display_rects()
 
@@ -271,7 +264,7 @@ class MeasureFrame(InvincibleFrame):
 			setcfg("dimensions.measureframe", self.get_dimensions())
 			if self.Parent and hasattr(self.Parent, "get_set_display"):
 				self.Parent.get_set_display()
-		wx.CallAfter(wx.Frame.Show, self, show)
+		wx.Frame.Show(self, show)
 
 	def Hide(self):
 		self.Show(False)
@@ -308,7 +301,7 @@ class MeasureFrame(InvincibleFrame):
 		if debug: safe_print("[D]  display_client_rect:", display_client_rect)
 		display_client_size = display_client_rect[2:]
 		if debug: safe_print("[D]  display_client_size:", display_client_size)
-		measureframe_min_size = list(self.GetMinSize())
+		measureframe_min_size = [max(self.sizer.GetMinSize())] * 2
 		if debug: safe_print("[D]  measureframe_min_size:", measureframe_min_size)
 		default_measureframe_size = get_default_size()
 		size = [min(display_client_size[0], 
@@ -324,7 +317,10 @@ class MeasureFrame(InvincibleFrame):
 		if max(size) >= max(display_client_size):
 			scale = 50
 		if debug: safe_print("[D]  measureframe_size:", size)
+		self.SetMaxSize((-1, -1))
+		self.SetMinSize(size)
 		self.SetSize(size)
+		self.SetMaxSize(size)
 		display_rect = display[1]
 		if debug: safe_print("[D]  display_rect:", display_rect)
 		display_size = display_rect[2:]
@@ -345,7 +341,7 @@ class MeasureFrame(InvincibleFrame):
 			measureframe_pos[1] = display_client_rect[1]
 		if debug: safe_print("[D]  measureframe_pos:", measureframe_pos)
 		setcfg("dimensions.measureframe", ",".join(strlist((x, y, scale))))
-		wx.CallAfter(self.SetPosition, measureframe_pos)
+		self.SetPosition(measureframe_pos)
 
 	def zoomin_handler(self, event):
 		if debug: safe_print("[D] measureframe_zoomin_handler")
@@ -466,34 +462,10 @@ class MeasureFrame(InvincibleFrame):
 		if display_no != self.display_no:
 			self.display_no = display_no
 			# Translate from wx display index to Argyll display index
-			geometry = "%i, %i, %ix%i" % tuple(geometry)
-			for i, display in enumerate(getcfg("displays").split(os.pathsep)):
-				if display.find("@ " + geometry) > -1:
-					if debug:
-						safe_print("[D] Found display %s at index %i" % 
-								   (geometry, i))
-					# Save Argyll display index to configuration
-					setcfg("display.number", i + 1)
-					break
-
-	def size_handler(self, event):
-		if debug: safe_print("[D] measureframe_size_handler")
-		display_client_size = self.get_display()[2][2:]
-		size = self.GetSize()
-		if debug:
-			safe_print("[D]  display_client_size:", display_client_size)
-			safe_print("[D]  measureframe_size:", size)
-			measureframe_pos = self.GetScreenPosition()
-			safe_print("[D]  measureframe_pos:", measureframe_pos)
-		if min(size) < min(display_client_size) - 50 and \
-		   size[0] != size[1]:
-			if sys.platform != "win32":
-				wx.CallAfter(self.SetSize, (min(size), 
-											min(size)))
-			else:
-				self.SetSize((min(size), min(size)))
-			if debug: wx.CallAfter(self.get_dimensions)
-		event.Skip()
+			n = get_argyll_display_number(geometry)
+			if n is not None:
+				# Save Argyll display index to configuration
+				setcfg("display.number", n + 1)
 
 	def get_dimensions(self):
 		"""
