@@ -115,7 +115,7 @@ import  wx
 
 # Needs Numeric or numarray or NumPy
 try:
-    import oldnumeric as _Numeric
+    import numpy as _Numeric
 except:
     try:
         import numarray as _Numeric  #if numarray is used it is renamed Numeric
@@ -130,6 +130,10 @@ except:
             Numeric Python site (http://numpy.scipy.org) for information on
             downloading source or binaries."""
             raise ImportError, "Numeric,numarray or NumPy not found. \n" + msg
+else:
+    _Numeric.Float = _Numeric.float
+    _Numeric.Float64 = _Numeric.float64
+    _Numeric.Int32 = _Numeric.int32
 
 
 
@@ -393,14 +397,14 @@ class PolyMarker(PolyPoints):
 
     def _triangle(self, dc, coords, size=1):
         shape= [(-2.5*size,1.44*size), (2.5*size,1.44*size), (0.0,-2.88*size)]
-        poly= _Numeric.repeat(coords,3)
+        poly= _Numeric.repeat(coords,3,0)
         poly.shape= (len(coords),3,2)
         poly += shape
         dc.DrawPolygonList(poly.astype(_Numeric.Int32))
 
     def _triangle_down(self, dc, coords, size=1):
         shape= [(-2.5*size,-1.44*size), (2.5*size,-1.44*size), (0.0,2.88*size)]
-        poly= _Numeric.repeat(coords,3)
+        poly= _Numeric.repeat(coords,3,0)
         poly.shape= (len(coords),3,2)
         poly += shape
         dc.DrawPolygonList(poly.astype(_Numeric.Int32))
@@ -1142,13 +1146,15 @@ class PlotCanvas(wx.Panel):
         elif self._pointSize != (1.0, 1.0):
             self._pointSize = (1.0, 1.0)
             self._setSize()
-        if sys.platform in ("darwin", "win32") or not isinstance(dc, wx.GCDC):
+        if (sys.platform in ("darwin", "win32") or
+            not isinstance(dc, wx.GCDC) or wx.VERSION >= (2, 9)):
             self._fontScale = sum(self._pointSize) / 2.0
         else:
             # on Linux, we need to correct the font size by a certain factor if wx.GCDC is used,
             # to make text the same size as if wx.GCDC weren't used
+            screenppi = map(float, wx.ScreenDC().GetPPI())
             ppi = dc.GetPPI()
-            self._fontScale = (96.0 / ppi[0] * self._pointSize[0] + 96.0 / ppi[1] * self._pointSize[1]) / 2.0
+            self._fontScale = (screenppi[0] / ppi[0] * self._pointSize[0] + screenppi[1] / ppi[1] * self._pointSize[1]) / 2.0
         graphics._pointSize = self._pointSize
             
         dc.SetTextForeground(self.GetForegroundColour())
@@ -1181,12 +1187,17 @@ class PlotCanvas(wx.Panel):
         # Get ticks and textExtents for axis if required
         if self._xSpec is not 'none':        
             xticks = self._xticks(xAxis[0], xAxis[1])
-            xTextExtent = dc.GetTextExtent(xticks[-1][1])# w h of x axis text last number on axis
         else:
             xticks = None
+        if xticks:
+            xTextExtent = dc.GetTextExtent(xticks[-1][1])# w h of x axis text last number on axis
+        else:
             xTextExtent= (0,0) # No text for ticks
         if self._ySpec is not 'none':
             yticks = self._yticks(yAxis[0], yAxis[1])
+        else:
+            yticks = None
+        if yticks:
             if self.getLogScale()[1]:
                 yTextExtent = dc.GetTextExtent('-2e-2')
             else:
@@ -1195,7 +1206,6 @@ class PlotCanvas(wx.Panel):
                 yTextExtent= (max(yTextExtentBottom[0],yTextExtentTop[0]),
                               max(yTextExtentBottom[1],yTextExtentTop[1]))
         else:
-            yticks = None
             yTextExtent= (0,0) # No text for ticks
 
         # TextExtents for Title and Axis Labels
@@ -1247,6 +1257,8 @@ class PlotCanvas(wx.Panel):
         ptx,pty,rectWidth,rectHeight= self._point2ClientCoord(p1, p2)
         # allow graph to overlap axis lines by adding units to width and height
         dc.SetClippingRegion(ptx*self._pointSize[0],pty*self._pointSize[1],rectWidth*self._pointSize[0]+2,rectHeight*self._pointSize[1]+1)
+        # Draw extras
+        self._drawExtras(dc, p1, p2, scale, shift)
         # Draw the lines and markers
         #start = _time.clock()
         graphics.draw(dc)
@@ -1715,23 +1727,27 @@ class PlotCanvas(wx.Panel):
                 dc.DrawLine(a1[0],a1[1],a2[0],a2[1])
                 text = 0    # axis values not drawn on right side
 
+    def _drawExtras(self, dc, p1, p2, scale, shift):
+        b1, b2 = self.last_draw[0].boundingBox()
+
         if self._centerLinesEnabled:
+            x, y = [b1[i] + (b2[i] - b1[i]) / 2.0 for i in xrange(2)]
+            x, y = self.PositionUserToScreen((x, y))
+            x, y = x * self._pointSize[0], y * self._pointSize[1]
             if self._centerLinesEnabled in ('Horizontal', True):
-                y1 = scale[1]*p1[1]+shift[1]
-                y2 = scale[1]*p2[1]+shift[1]
-                y = (y1 - y2) / 2.0 + y2
                 dc.DrawLine(scale[0] * p1[0] + shift[0], y, scale[0] * p2[0] + shift[0], y)
             if self._centerLinesEnabled in ('Vertical', True):
-                x1 = scale[0]*p1[0]+shift[0]
-                x2 = scale[0]*p2[0]+shift[0]
-                x = (x1 - x2) / 2.0 + x2
                 dc.DrawLine(x, scale[1] * p1[1] + shift[1], x, scale[1] * p2[1] + shift[1])
 
         if self._diagonalsEnabled:
+            x1, y1 = self.PositionUserToScreen(b1)
+            x1, y1 = x1 * self._pointSize[0], y1 * self._pointSize[1]
+            x2, y2 = self.PositionUserToScreen(b2)
+            x2, y2 = x2 * self._pointSize[0], y2 * self._pointSize[1]
             if self._diagonalsEnabled in ('Bottomleft-Topright', True):
-                dc.DrawLine(scale[0] * p1[0] + shift[0], scale[1] * p1[1] + shift[1], scale[0] * p2[0] + shift[0], scale[1] * p2[1] + shift[1])
+                dc.DrawLine(x1, y1, x2, y2)
             if self._diagonalsEnabled in ('Bottomright-Topleft', True):
-                dc.DrawLine(scale[0] * p1[0] + shift[0], scale[1] * p2[1] + shift[1], scale[0] * p2[0] + shift[0], scale[1] * p1[1] + shift[1])
+                dc.DrawLine(x1, y2, x2, y1)
 
     def _xticks(self, *args):
         if self._logscale[0]:
