@@ -6,18 +6,17 @@ import os
 import sys
 
 import config
-from config import (btn_width_correction, defaults, getbitmap, getcfg, geticon, 
-					get_bitmap_as_icon, get_data_path, get_verified_path, 
-					setcfg)
-from debughelpers import getevtobjname, getevttype
+from config import (defaults, getbitmap, getcfg, geticon, 
+					get_verified_path, setcfg)
+from debughelpers import getevtobjname, getevttype, handle_error
 from log import log as log_, safe_print
 from meta import name as appname
 from options import debug
-from thread import start_new_thread
 from util_os import waccess
 from util_str import safe_unicode, wrap
 from wxaddons import (CustomEvent, FileDrop as _FileDrop, get_dc_font_size,
-					  get_platform_window_decoration_size, wx)
+					  get_platform_window_decoration_size, wx,
+					  BetterWindowDisabler)
 from wxfixes import GTKMenuItemGetFixedLabel
 from lib.agw import labelbook
 from lib.agw.gradientbutton import GradientButton, HOVER
@@ -98,8 +97,9 @@ class BaseFrame(wx.Frame):
 		if debug and hasattr(self, "last_focused_ctrl"):
 				safe_print("[D] Last focused control: %s" %
 						   self.last_focused_ctrl)
-		if hasattr(self, "last_focused_ctrl") and self.last_focused_ctrl and \
-		   self.last_focused_ctrl != event.GetEventObject():
+		if (hasattr(self, "last_focused_ctrl") and self.last_focused_ctrl and
+			self.last_focused_ctrl != event.GetEventObject() and
+			self.last_focused_ctrl.IsShownOnScreen()):
 			catchup_event = wx.FocusEvent(wx.EVT_KILL_FOCUS.evtType[0], 
 										  self.last_focused_ctrl.GetId())
 			if debug:
@@ -119,8 +119,9 @@ class BaseFrame(wx.Frame):
 					event = CustomEvent(event.GetEventType(), 
 										event.GetEventObject(), 
 										self.last_focused_ctrl)
-		if hasattr(event.GetEventObject(), "GetId") and \
-		   callable(event.GetEventObject().GetId):
+		if (hasattr(event.GetEventObject(), "GetId") and
+			callable(event.GetEventObject().GetId) and
+			event.GetEventObject().IsShownOnScreen()):
 		   	if debug:
 					safe_print("[D] Setting last focused control to %s " %
 							   event.GetEventObject())
@@ -251,12 +252,10 @@ class BaseInteractiveDialog(wx.Dialog):
 	
 	def __init__(self, parent=None, id=-1, title=appname, msg="", 
 				 ok="OK", bitmap=None, pos=(-1, -1), size=(400, -1), 
-				 show=True, log=True, print_=False, 
+				 show=True, log=True, 
 				 style=wx.DEFAULT_DIALOG_STYLE, nowrap=False, wrap=70):
-		if print_:
-			safe_print(msg, log=False)
 		if log:
-			log_(msg)
+			safe_print(msg)
 		if parent:
 			pos = list(pos)
 			i = 0
@@ -299,8 +298,6 @@ class BaseInteractiveDialog(wx.Dialog):
 		btnwidth = 80
 
 		self.ok = wx.Button(self, wx.ID_OK, ok)
-		self.ok.SetInitialSize((self.ok.GetSize()[0] + btn_width_correction, 
-		   -1))
 		self.sizer2.Add(self.ok)
 		self.Bind(wx.EVT_BUTTON, self.OnClose, id=wx.ID_OK)
 
@@ -419,6 +416,8 @@ class BitmapBackgroundPanelText(BitmapBackgroundPanel):
 	def __init__(self, *args, **kwargs):
 		BitmapBackgroundPanel.__init__(self, *args, **kwargs)
 		self._label = ""
+		self.drawline = True
+		self.textshadowcolor = wx.Colour(214, 214, 214)
 	
 	def _set_font(self, dc):
 		try:
@@ -438,10 +437,11 @@ class BitmapBackgroundPanelText(BitmapBackgroundPanel):
 	
 	def _draw(self, dc):
 		BitmapBackgroundPanel._draw(self, dc)
-		pen = wx.Pen(wx.Colour(0x66, 0x66, 0x66), 1, wx.SOLID)
-		pen.SetCap(wx.CAP_BUTT)
-		dc.SetPen(pen)
-		dc.DrawLine(0, self.GetSize()[1] - 1, self.GetSize()[0], self.GetSize()[1] - 1)
+		if self.drawline:
+			pen = wx.Pen(wx.Colour(0x66, 0x66, 0x66), 1, wx.SOLID)
+			pen.SetCap(wx.CAP_BUTT)
+			dc.SetPen(pen)
+			dc.DrawLine(0, self.GetSize()[1] - 1, self.GetSize()[0], self.GetSize()[1] - 1)
 		dc = self._set_font(dc)
 		w1, h1 = self.GetTextExtent(self.GetLabel())
 		w2, h2 = dc.GetTextExtent(self.GetLabel())
@@ -449,8 +449,9 @@ class BitmapBackgroundPanelText(BitmapBackgroundPanel):
 		h = (max(h1, h2) - min(h1, h2)) / 2.0 + min(h1, h2)
 		x, y = (self.GetSize()[0] / 2.0 - w / 2.0,
 				self.GetSize()[1] / 2.0 - h / 2.0)
-		dc.SetTextForeground(wx.Colour(214, 214, 214))
-		dc.DrawText(self.GetLabel(), x + 1, y + 1)
+		if self.textshadowcolor:
+			dc.SetTextForeground(self.textshadowcolor)
+			dc.DrawText(self.GetLabel(), x + 1, y + 1)
 		dc.SetTextForeground(self.GetForegroundColour())
 		dc.DrawText(self.GetLabel(), x, y)
 
@@ -461,11 +462,11 @@ class ConfirmDialog(BaseInteractiveDialog):
 
 	def __init__(self, parent=None, id=-1, title=appname, msg="", 
 				 ok="OK", cancel="Cancel", bitmap=None, pos=(-1, -1), 
-				 size=(400, -1), alt=None, log=False, print_=False, 
+				 size=(400, -1), alt=None, log=False, 
 				 style=wx.DEFAULT_DIALOG_STYLE, nowrap=False, wrap=70):
 		BaseInteractiveDialog.__init__(self, parent, id, title, msg, ok, 
 									   bitmap, pos, size, show=False, 
-									   log=log, print_=print_, style=style,
+									   log=log, style=style,
 									   nowrap=nowrap, wrap=wrap)
 
 		self.Bind(wx.EVT_CLOSE, self.OnClose, self)
@@ -474,15 +475,11 @@ class ConfirmDialog(BaseInteractiveDialog):
 
 		if alt:
 			self.alt = wx.Button(self, -1, alt)
-			self.alt.SetInitialSize((self.alt.GetSize()[0] + 
-			   btn_width_correction, -1))
 			self.sizer2.Prepend((margin, margin))
 			self.sizer2.Prepend(self.alt)
 			self.Bind(wx.EVT_BUTTON, self.OnClose, id=self.alt.GetId())
 
 		self.cancel = wx.Button(self, wx.ID_CANCEL, cancel)
-		self.cancel.SetInitialSize((self.cancel.GetSize()[0] + 
-		   btn_width_correction, -1))
 		self.sizer2.Prepend((margin, margin))
 		self.sizer2.Prepend(self.cancel)
 		self.Bind(wx.EVT_BUTTON, self.OnClose, id=wx.ID_CANCEL)
@@ -491,7 +488,10 @@ class ConfirmDialog(BaseInteractiveDialog):
 
 	def OnClose(self, event):
 		if hasattr(self, "OnCloseIntercept"):
-			self.OnCloseIntercept(event)
+			try:
+				self.OnCloseIntercept(event)
+			except Exception, exception:
+				handle_error(exception, self)
 			return
 		if event.GetEventObject() == self:
 			id = wx.ID_CANCEL
@@ -604,6 +604,9 @@ class FileBrowseBitmapButtonWithChoiceHistory(filebrowse.FileBrowseButtonWithHis
 			history = self.history
 			self.history = []
 			self.SetHistory(history)
+		if sys.platform == "darwin" and wx.VERSION > (2, 9):
+			# Prevent 1px cut-off at left-hand side
+			box.Add((1, 1))
 		box.Add(self.textControl, 1, wx.ALIGN_CENTER_VERTICAL | wx.TOP |
 									 wx.BOTTOM, 4)
 
@@ -666,9 +669,6 @@ class FlatShadedButton(GradientButton):
 				 name="gradientbutton", bgcolour=None, fgcolour=None):
 		GradientButton.__init__(self, parent, id, bitmap, label, pos, size,
 								style, validator, name)
-		if sys.platform != "win32":
-			self.SetFont(wx.Font(11, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL,
-									 wx.FONTWEIGHT_NORMAL))
 		self._setcolours(bgcolour, fgcolour)
 	
 	def _setcolours(self, bgcolour=None, fgcolour=None):
@@ -754,9 +754,9 @@ class InfoDialog(BaseInteractiveDialog):
 
 	def __init__(self, parent=None, id=-1, title=appname, msg="", 
 				 ok="OK", bitmap=None, pos=(-1, -1), size=(400, -1), 
-				 show=True, log=True, print_=False):
+				 show=True, log=True):
 		BaseInteractiveDialog.__init__(self, parent, id, title, msg, ok, 
-									   bitmap, pos, size, show, log, print_)
+									   bitmap, pos, size, show, log)
 
 
 class InvincibleFrame(wx.Frame):
@@ -910,7 +910,8 @@ class ProgressDialog(wx.Dialog):
 				 handler=None, keyhandler=None, start_timer=True, pos=None,
 				 pauseable=False):
 		if style is None:
-			style = wx.PD_APP_MODAL | wx.PD_ELAPSED_TIME | wx.PD_CAN_ABORT | wx.PD_SMOOTH
+			style = (wx.PD_APP_MODAL | wx.PD_ELAPSED_TIME |
+					 wx.PD_REMAINING_TIME | wx.PD_CAN_ABORT | wx.PD_SMOOTH)
 		wx.Dialog.__init__(self, parent, wx.ID_ANY, title)
 		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16], appname))
 		self.Bind(wx.EVT_CLOSE, self.OnClose, self)
@@ -944,18 +945,29 @@ class ProgressDialog(wx.Dialog):
 		self.gauge = wx.Gauge(self, wx.ID_ANY, range=maximum, style=gauge_style)
 		self.sizer1.Add(self.gauge, flag=wx.EXPAND | wx.BOTTOM, border=margin)
 		
-		if style & wx.PD_ELAPSED_TIME:
-			self.sizer3 = wx.BoxSizer(wx.HORIZONTAL)
+		if style & wx.PD_ELAPSED_TIME or style & wx.PD_REMAINING_TIME:
+			self.sizer3 = wx.FlexGridSizer(0, 2, 0, margin)
 			self.sizer1.Add(self.sizer3, flag=wx.ALIGN_CENTER)
+			self.time = time()
+		
+		if style & wx.PD_ELAPSED_TIME:
 			self.sizer3.Add(wx.StaticText(self, -1,
 										  lang.getstr("elapsed_time")))
-			self.time = time()
-			self.elapsed_time = wx.StaticText(self, -1, "")
+			self.elapsed_time = wx.StaticText(self, -1, "--:--:--")
 			self.elapsed_time_handler(None)
-			self.sizer3.Add(self.elapsed_time, flag=wx.LEFT, border=margin)
+			self.sizer3.Add(self.elapsed_time)
 			self.elapsed_timer = wx.Timer(self)
 			self.Bind(wx.EVT_TIMER, self.elapsed_time_handler,
 					  self.elapsed_timer)
+		
+		if style & wx.PD_REMAINING_TIME:
+			self.sizer3.Add(wx.StaticText(self, -1,
+										  lang.getstr("remaining_time")))
+			self.time2 = 0
+			self.time3 = time()
+			self.time4 = 0
+			self.remaining_time = wx.StaticText(self, -1, u"––:––:––")
+			self.sizer3.Add(self.remaining_time)
 
 		if style & wx.PD_CAN_ABORT:
 			self.cancel = wx.Button(self, wx.ID_ANY, lang.getstr("cancel"))
@@ -1016,6 +1028,14 @@ class ProgressDialog(wx.Dialog):
 		if style & wx.PD_APP_MODAL:
 			self.MakeModal()
 	
+	def MakeModal(self, modal=True):
+		# wxPython 3.0 deprecates MakeModal, use a replacement implementation
+		# based on http://wxpython.org/Phoenix/docs/html/MigrationGuide.html
+		if modal and not hasattr(self, '_disabler'):
+			self._disabler = BetterWindowDisabler(self)
+		if not modal and hasattr(self, '_disabler'):
+			del self._disabler
+	
 	def OnClose(self, event):
 		self.keepGoing = False
 		if hasattr(self, "cancel"):
@@ -1061,6 +1081,11 @@ class ProgressDialog(wx.Dialog):
 			self.msg.SetLabel(msg)
 			self.msg.Refresh()
 			self.msg.Update()
+		if getattr(self, "time2", 0):
+			self.time2 = 0
+			if not self.time3:
+				self.time3 = time()
+			self.remaining_time.Label = u"––:––:––"
 		self.gauge.Pulse()
 		return self.keepGoing, self.skip
 	
@@ -1068,13 +1093,33 @@ class ProgressDialog(wx.Dialog):
 		self.keepGoing = True
 		if hasattr(self, "cancel"):
 			self.cancel.Enable()
-		self.pause_continue.Enable()
+		if self.paused:
+			self.pause_continue_handler()
+		else:
+			self.pause_continue.Enable()
 	
 	def Update(self, value, msg=None):
 		if msg and msg != self.msg.Label:
 			self.msg.SetLabel(msg)
 			self.msg.Refresh()
 			self.msg.Update()
+		if hasattr(self, "time2"):
+			t = time()
+			if not self.time2:
+				if value < self.gauge.GetValue():
+					# Reset
+					self.time2 = t
+				else:
+					# Continue
+					self.time4 += time() - self.time3
+					self.time2 = self.time + self.time4
+					self.time3 = 0
+			if value and value != self.gauge.GetValue() and self.time2 < t:
+				remaining = ((t - self.time2) / value *
+								  (self.gauge.GetRange() - value))
+				if remaining > 9 or value > self.gauge.GetRange() * .03:
+					self.remaining_time.Label = strftime("%H:%M:%S",
+														 gmtime(remaining))
 		self.gauge.SetValue(value)
 		return self.keepGoing, self.skip
 	
@@ -1313,6 +1358,7 @@ class TooltipWindow(InvincibleFrame):
 			self.sizer1.Add(self.bitmap, flag=wx.RIGHT, border=margin)
 
 		self.message = wx.StaticText(self.panel, -1, util_str.wrap(msg, wrap))
+		self.message.SetMaxFontSize()
 		self.sizer1.Add(self.message)
 
 		self.sizer0.SetSizeHints(self)
@@ -1365,7 +1411,7 @@ class TwoWaySplitter(FourWaySplitter):
 			totw = width - barSize - 2*border
 			self._splitx = max((self._fhor*totw)/10000, self._minimum_pane_size)
 			self._splity = height
-			rightw = totw - self._splitx
+			rightw = max(totw - self._splitx, 0)
 			if win0:
 				win0.SetDimensions(0, 0, self._splitx, self._splity)
 				win0.Show()
